@@ -5,7 +5,7 @@
  * libradius.h	Structures and prototypes
  *		for the radius library.
  *
- * Version:	$Id: libradius.h,v 1.46 2001/11/08 01:12:32 cparker Exp $
+ * Version:	$Id: libradius.h,v 1.50 2002/03/06 21:15:37 aland Exp $
  *
  */
 
@@ -59,8 +59,8 @@
 #define CHAP_VALUE_LENGTH       16
 #define MAX_STRING_LEN		254	/* RFC2138: string 0-253 octets */
 
-#define PW_AUTH_UDP_PORT                1645
-#define PW_ACCT_UDP_PORT                1646
+#define PW_AUTH_UDP_PORT                1812
+#define PW_ACCT_UDP_PORT                1813
 
 #ifdef _LIBRADIUS
 #  define AUTH_HDR_LEN		20
@@ -202,7 +202,7 @@ void		rad_free(RADIUS_PACKET **);
 int		rad_pwencode(char *encpw, int *len, const char *secret, const char *vector);
 int		rad_pwdecode(char *encpw, int len, const char *secret, const char *vector);
 int		rad_tunnel_pwencode(char *encpw, int *len, const char *secret, const char *vector);
-int		rad_tunnel_pwdecode(char *encpw, int len, const char *secret, const char *vector);
+int		rad_tunnel_pwdecode(char *encpw, int *len, const char *secret, const char *vector);
 int		rad_chap_encode(RADIUS_PACKET *packet, char *output, int id, VALUE_PAIR *password);
 
 /* valuepair.c */
@@ -235,8 +235,9 @@ void		librad_perror(const char *, ...)
 #endif
 ;
 extern char	librad_errstr[];
-extern int	librad_dodns;
-extern int	librad_debug;
+extern int	librad_dodns;	/* 0 = no dns lookups */
+extern int	librad_debug;	/* 0 = no debugging information */
+extern int	librad_max_attributes; /* per incoming packet */
 
 /*
  *	Several handy miscellaneous functions.
@@ -248,6 +249,8 @@ uint32_t	ip_addr(const char *);
 char		*strNcpy(char *dest, const char *src, int n);
 void		rad_lowercase(char *str);
 void		rad_rmspace(char *str);
+int		rad_lockfd(int fd, int lock_len);
+int		rad_unlockfd(int fd, int lock_len);
 
 #ifdef ASCEND_BINARY
 /* filters.c */
