@@ -4,7 +4,7 @@
  * radiusd.h	Structures, prototypes and global variables
  *		for the FreeRADIUS server.
  *
- * Version:	$Id: radiusd.h,v 1.95 2001/11/27 22:44:24 kkalev Exp $
+ * Version:	$Id: radiusd.h,v 1.100 2002/03/02 05:50:43 ramoore Exp $
  *
  */
 #include "libradius.h"
@@ -64,8 +64,9 @@ typedef struct auth_req {
 	void			**container;
 } REQUEST;
 
-#define RAD_REQUEST_OPTION_NONE         (0)
-#define RAD_REQUEST_OPTION_LOGGED_CHILD (1 << 0)
+#define RAD_REQUEST_OPTION_NONE            (0)
+#define RAD_REQUEST_OPTION_LOGGED_CHILD    (1 << 0)
+#define RAD_REQUEST_OPTION_DELAYED_REJECT  (1 << 1)
 
 /*
  *  Function handler for requests.
@@ -185,11 +186,11 @@ extern uint32_t		myip;
 extern int		log_auth_detail;
 extern int		auth_port;
 extern int		acct_port;
-extern int		acctfd;
 extern int		proxy_port;
 extern int		proxyfd;
 extern int		proxy_retry_count;
 extern int		proxy_retry_delay;
+extern int		proxy_fallback;
 extern int		spawn_flag;
 extern const char      *radiusd_version;
 
@@ -206,7 +207,7 @@ int		rad_accounting(REQUEST *);
 /* session.c */
 int		rad_check_ts(uint32_t nasaddr, int port, const char *user,
 			     const char *sessionid);
-int		session_zap(uint32_t nasaddr, int port, const char *user,
+int		session_zap(int fd, uint32_t nasaddr, int port, const char *user,
 			    const char *sessionid, uint32_t cliaddr,
 			    char proto, time_t t);
 
@@ -225,6 +226,7 @@ int		rad_mkdir(char *directory, int mode);
 int		rad_checkfilename(const char *filename);
 void		*rad_malloc(size_t size); /* calls exit(1) on error! */
 void		xfree(const char *ptr);
+void		rad_assert_fail (const char *file, unsigned int line);
 
 /* client.c */
 int		read_clients_file(const char *file);
@@ -246,13 +248,14 @@ int		read_naslist_file(char *);
 NAS		*nas_find(uint32_t ipno);
 const char	*nas_name(uint32_t ipno);
 const char	*nas_name2(RADIUS_PACKET *r);
+char  *		nas_name3(char *buf, size_t buflen, uint32_t ipno);
 NAS		*nas_findbyname(char *nasname);
 
 /* version.c */
 void		version(void);
 
 /* log.c */
-int radlogdir_iswritable(char *);
+int radlogdir_iswritable(const char *);
 int		radlog(int, const char *, ...)
 #ifdef __GNUC__
 		__attribute__ ((format (printf, 2, 3)))
@@ -302,9 +305,7 @@ int		xlat_register(char *module, RAD_XLAT_FUNC func, void *instance);
 void		xlat_unregister(char *module, RAD_XLAT_FUNC func);
 
 
-#ifdef WITH_THREAD_POOL
 /* threads.c */
 extern		int thread_pool_init(void);
 extern		int thread_pool_clean(time_t now);
-#endif
 #endif /*RADIUSD_H*/
