@@ -12,7 +12,7 @@
  *	FOR A PARTICULAR PURPOSE.
  */
 
-/* $Id: filters.c,v 1.20 2001/08/28 20:13:37 aland Exp $ */
+/* $Id: filters.c,v 1.22 2002/03/02 06:49:55 ramoore Exp $ */
 
 /*
  *  Alan DeKok's comments:  This code SUCKS.  Having a static string
@@ -1339,6 +1339,12 @@ void print_abinary(VALUE_PAIR *vp, u_char *buffer, int len)
       len -= i;
     }
 
+    if (filter.u.ip.established) {
+      i = snprintf(p, len, " est");
+      p += i;
+      len -= i;
+    }
+
     /*
      *	Handle IPX filters
      */
@@ -1346,7 +1352,7 @@ void print_abinary(VALUE_PAIR *vp, u_char *buffer, int len)
     /* print for source */
     if (filter.u.ipx.srcIpxNet) {
       i = snprintf(p, len, " srcipxnet 0x%04x srcipxnode 0x%02x%02x%02x%02x%02x%02x",
-		  ntohl(filter.u.ipx.srcIpxNet),
+		  (unsigned int)ntohl(filter.u.ipx.srcIpxNet),
 		  filter.u.ipx.srcIpxNode[0], filter.u.ipx.srcIpxNode[1], 
 		  filter.u.ipx.srcIpxNode[2], filter.u.ipx.srcIpxNode[3], 
 		  filter.u.ipx.srcIpxNode[4], filter.u.ipx.srcIpxNode[5]);
@@ -1365,7 +1371,7 @@ void print_abinary(VALUE_PAIR *vp, u_char *buffer, int len)
     /* same for destination */
     if (filter.u.ipx.dstIpxNet) {
       i = snprintf(p, len, " dstipxnet 0x%04x dstipxnode 0x%02x%02x%02x%02x%02x%02x",
-		  ntohl(filter.u.ipx.dstIpxNet),
+		  (unsigned int)ntohl(filter.u.ipx.dstIpxNet),
 		  filter.u.ipx.dstIpxNode[0], filter.u.ipx.dstIpxNode[1], 
 		  filter.u.ipx.dstIpxNode[2], filter.u.ipx.dstIpxNode[3], 
 		  filter.u.ipx.dstIpxNode[4], filter.u.ipx.dstIpxNode[5]);
