@@ -1,11 +1,11 @@
 /*
  * misc.c	Various miscellaneous functions.
  *
- * Version:	$Id: misc.c,v 1.22 2001/10/10 18:03:49 aland Exp $
+ * Version:	$Id: misc.c,v 1.28 2002/03/13 16:48:39 aland Exp $
  */
 
 static const char rcsid[] =
-"$Id: misc.c,v 1.22 2001/10/10 18:03:49 aland Exp $";
+"$Id: misc.c,v 1.28 2002/03/13 16:48:39 aland Exp $";
 
 #include	"autoconf.h"
 
@@ -19,7 +19,9 @@ static const char rcsid[] =
 #include	<string.h>
 #include	<netdb.h>
 #include	<ctype.h>
-#include	<signal.h>
+#include	<sys/file.h>
+#include	<fcntl.h>
+#include	<unistd.h>
 
 #include	"libradius.h"
 #include	"missing.h"
@@ -34,17 +36,32 @@ int		librad_debug = 0;
 char * ip_hostname(char *buf, size_t buflen, uint32_t ipaddr)
 {
 	struct		hostent *hp;
+#ifdef GETHOSTBYADDRRSTYLE
+	char buffer[2048];
+	struct hostent result;
+	int error;
+#endif
 
 	/*
 	 *	No DNS: don't look up host names
 	 */
-	if (!librad_dodns) {
+	if (librad_dodns == 0) {
 		ip_ntoa(buf, ipaddr);
 		return buf;
 	}
 
-	hp = gethostbyaddr((char *)&ipaddr, sizeof (struct in_addr), AF_INET);
-	if ((hp == 0) ||
+#ifdef GETHOSTBYADDRRSTYLE
+#if GETHOSTBYADDRRSTYLE == SYSVSTYLE
+	hp = gethostbyaddr_r((char *)&ipaddr, sizeof(struct in_addr), AF_INET, &result, buffer, sizeof(buffer), &error);
+#elif GETHOSTBYADDRRSTYLE == GNUSTYLE
+	gethostbyaddr_r((char *)&ipaddr, sizeof(struct in_addr), AF_INET, &result, buffer, sizeof(buffer), &hp, &error);
+#else
+	hp = gethostbyaddr((char *)&ipaddr, sizeof(struct in_addr), AF_INET);
+#endif
+#else
+	hp = gethostbyaddr((char *)&ipaddr, sizeof(struct in_addr), AF_INET);
+#endif
+	if ((hp == NULL) ||
 	    (strlen((char *)hp->h_name) >= buflen)) {
 		ip_ntoa(buf, ipaddr);
 		return buf;
@@ -63,12 +80,23 @@ uint32_t ip_getaddr(const char *host)
 {
 	struct hostent	*hp;
 	uint32_t	 a;
+#ifdef HAVE_GETHOSTBYNAME_R
+	struct hostent result;
+	int error;
+	char buffer[2048];
+#endif
 
 	if ((a = ip_addr(host)) != htonl(INADDR_NONE))
 		return a;
 
-	if ((hp = gethostbyname(host)) == NULL)
+#ifndef HAVE_GETHOSTBYNAME_R
+	hp = gethostbyname(host);
+#else
+	hp = gethostbyname_r(host, &result, buffer, sizeof(buffer), &error);
+#endif
+	if (hp == NULL) {
 		return htonl((uint32_t) INADDR_NONE);
+	}
 
 	/*
 	 *	Paranoia from a Bind vulnerability.  An attacker
@@ -155,4 +183,31 @@ void rad_rmspace(char *str) {
   *s = '\0';
 }
 
+/*
+ *	Internal wrapper for locking, to minimize the number of ifdef's
+ *
+ *	Lock an fd, prefer lockf() over flock()
+ */
+int rad_lockfd(int fd, int lock_len)
+{
+#if defined(F_LOCK) && !defined(BSD)
+	return lockf(fd, F_LOCK, lock_len);
+#else
+	return flock(fd, LOCK_EX);
+#endif
+}
 
+/*
+ *	Internal wrapper for unlocking, to minimize the number of ifdef's
+ *	in the source.
+ *
+ *	Unlock an fd, prefer lockf() over flock()
+ */
+int rad_unlockfd(int fd, int lock_len)
+{
+#if defined(F_LOCK) && !defined(BSD)
+	return lockf(fd, F_ULOCK, lock_len);
+#else
+	return flock(fd, LOCK_UN);
+#endif
+}
