@@ -1,11 +1,11 @@
 /*
  * radius.c	Functions to send/receive radius packets.
  *
- * Version:	$Id: radius.c,v 1.63 2001/11/29 09:45:00 3APA3A Exp $
+ * Version:	$Id: radius.c,v 1.69 2002/02/20 16:19:13 aland Exp $
  *
  */
 
-static const char rcsid[] = "$Id: radius.c,v 1.63 2001/11/29 09:45:00 3APA3A Exp $";
+static const char rcsid[] = "$Id: radius.c,v 1.69 2002/02/20 16:19:13 aland Exp $";
 
 #include	"autoconf.h"
 #include	"md5.h"
@@ -116,7 +116,7 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 	 */
 	if (!packet->data) {
 		  radius_packet_t	*hdr;
-		  int32_t		lvalue;
+		  uint32_t		lvalue;
 		  uint8_t		*ptr, *length_ptr, *vsa_length_ptr;
 		  uint8_t		digest[16];
 		  int			secretlen;
@@ -322,8 +322,6 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 				          len++;
 					  if(TAG_VALID(reply->flags.tag)) {
 					          *ptr++ = reply->flags.tag;
-					  } else {
-					          *ptr++ = 0x00;
 					  }
 				  }
 				 
@@ -519,7 +517,7 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 	sa->sin_port = htons(packet->dst_port);
 
 	return sendto(packet->sockfd, packet->data, (int)packet->data_len, 0,
-		      &saremote, sizeof(struct sockaddr_in));
+		      (struct sockaddr *)&saremote, sizeof(struct sockaddr_in));
 }
 
 
@@ -642,6 +640,15 @@ RADIUS_PACKET *rad_recv(int fd)
 	packet->data_len = recvfrom(fd, data, sizeof(data),
 		0, (struct sockaddr *)&saremote, &salen);
 
+	/*
+	 *	Check for socket errors.
+	 */
+	if (packet->data_len < 0) {
+		librad_log("Error receiving packet: %s", strerror(errno));
+		free(packet);
+		return NULL;
+	}
+
 	/*
 	 *	Fill IP header fields.  We need these for the error
 	 *	messages which may come later.
@@ -655,17 +662,6 @@ RADIUS_PACKET *rad_recv(int fd)
 	 */
 	packet->vps = NULL;
 
-	/*
-	 *	Check for socket errors.
-	 */
-	if (packet->data_len < 0) {
-		librad_log("Error receiving packet from host %s: %s",
-			   ip_ntoa(host_ipaddr, packet->src_ipaddr),
-			   strerror(errno));
-		free(packet);
-		return NULL;
-	}
-
 	/*
 	 *	Check for packets smaller than the packet header.
 	 */
@@ -789,8 +785,7 @@ RADIUS_PACKET *rad_recv(int fd)
 			}
 			if (vendorlen != 0){
 				librad_log("WARNING: Malformed RADIUS packet from host %s: Vendor specific attributes do not exactly fill Vendor-Specific",
-				   ip_ntoa(host_ipaddr, packet->src_ipaddr),
-				   vendorattr[1] - 2);
+					   ip_ntoa(host_ipaddr, packet->src_ipaddr));
 				free(packet);
 				return NULL;
 			}
@@ -1099,12 +1094,25 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 				pair->length = strlen(pair->strvalue);
 			} else if (pair->flags.has_tag &&
 				   pair->type == PW_TYPE_STRING) {
-			        if(TAG_VALID(*ptr)) 
-				       pair->flags.tag = *ptr;
-				else
+				if(TAG_VALID(*ptr)) {
+				       pair->flags.tag = *ptr++;
+					pair->length--;
+				} else if(pair->flags.encrypt == 2) {
+					/*
+					 * from RFC2868 - 3.5.  Tunnel-Password
+					 * If the value of the Tag field is greater than
+					 * 0x00 and less than or equal to 0x1F, it SHOULD
+					 * be interpreted as indicating which tunnel
+					 * (of several alternatives) this attribute pertains;
+					 * otherwise, the Tag field SHOULD be ignored.
+					 */
+					pair->flags.tag = 0x00;
+					ptr++;
+					pair->length--;
+				} else {
 				       pair->flags.tag = 0x00;
-				pair->length--;
-				memcpy(pair->strvalue, ptr + 1, 
+				}
+				memcpy(pair->strvalue, ptr, 
 				       pair->length);
 			} else {
 				/* attrlen always < MAX_STRING_LEN */
@@ -1116,10 +1124,9 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 			if(pair->flags.encrypt == 2) {
 
 			        rad_tunnel_pwdecode((char *)pair->strvalue,
-						     pair->length, 
+						     &pair->length, 
 						     secret,
 						     (char *)original->vector);
-				pair->length = strlen(pair->strvalue);
 
 			}
 
@@ -1321,12 +1328,14 @@ int rad_pwdecode(char *passwd, int pwlen, const char *secret, const char *vector
 int rad_tunnel_pwencode(char *passwd, int *pwlen, const char *secret, const char *vector)
 {
 	uint8_t	buffer[AUTH_VECTOR_LEN + MAX_STRING_LEN + 3];
-	char	digest[AUTH_VECTOR_LEN];
+	unsigned char	digest[AUTH_VECTOR_LEN];
 	char    salt[2];
 	int	i, n, secretlen;
 	int	len;
+	
+	len = *pwlen;
 
-	if(pwlen < 2) {
+	if (len < 3) {
 	  return 0;
 	}
 	salt[0] = passwd[0];
@@ -1334,46 +1343,43 @@ int rad_tunnel_pwencode(char *passwd, int *pwlen, const char *secret, const char
 
 	/* Advance pointer past the salt, which is first two chars of passwd */
 	passwd = passwd + 2;
+	len -= 2;
+	*passwd = len;
 	
 	/*
 	 *	Padd password to multiple of AUTH_PASS_LEN bytes.
 	 */
-	len = strlen(passwd);
 	if (len > 128) len = 128;
-	*pwlen = len;
-	if (len % AUTH_PASS_LEN != 0) {
-		n = AUTH_PASS_LEN - (len % AUTH_PASS_LEN);
+	n = len % AUTH_PASS_LEN;
+	if (n) {
+		n = AUTH_PASS_LEN - n;
 		for (i = len; n > 0; n--, i++)
 			passwd[i] = 0;
-		len = *pwlen = i;
+		len = i;
 	}
+	
+	*pwlen = len+2;
+	
 
 	/*
 	 *	Use the secret to setup the decryption digest
 	 */
 	secretlen = strlen(secret);
 	memcpy(buffer, secret, secretlen);
-	memcpy(buffer + secretlen, vector, AUTH_VECTOR_LEN);
-	memcpy(buffer + secretlen + AUTH_VECTOR_LEN, salt, 2);
-	librad_md5_calc((u_char *)digest, buffer, secretlen + AUTH_VECTOR_LEN + 2);
-
-	/*
-	 *	Now we can encode the password *in place*
-	 */
-	for (i = 0; i < AUTH_PASS_LEN; i++)
-		passwd[i] ^= digest[i];
-
-	if (len <= AUTH_PASS_LEN) return 0;
-
-	/*
-	 *	Length > AUTH_PASS_LEN, so we need to use the extended
-	 *	algorithm.
-	 */
-	for (n = 0; n < 128 && n <= (len - AUTH_PASS_LEN); n += AUTH_PASS_LEN) { 
-		memcpy(buffer + secretlen, passwd + n, AUTH_PASS_LEN);
-		librad_md5_calc((u_char *)digest, buffer, secretlen + AUTH_PASS_LEN);
+	
+	for (n = 0; n < len; n+=AUTH_PASS_LEN) {
+		if (!n) {
+			memcpy(buffer + secretlen, vector, AUTH_VECTOR_LEN);
+			memcpy(buffer + secretlen + AUTH_VECTOR_LEN, salt, 2);
+			librad_md5_calc(digest, buffer, secretlen + AUTH_VECTOR_LEN + 2);
+		}
+		else {
+			memcpy(buffer + secretlen, passwd + n - AUTH_PASS_LEN, AUTH_PASS_LEN);
+			librad_md5_calc(digest, buffer, secretlen + AUTH_PASS_LEN);
+		}
+		
 		for (i = 0; i < AUTH_PASS_LEN; i++)
-			passwd[i + n + AUTH_PASS_LEN] ^= digest[i];
+			passwd[i + n] ^= digest[i];
 	}
 
 	return 0;
@@ -1386,62 +1392,54 @@ int rad_tunnel_pwencode(char *passwd, int *pwlen, const char *secret, const char
  *      value, to differentiate it from the above.
  */
 
-int rad_tunnel_pwdecode(char *passwd, int pwlen, const char *secret, const char *vector)
+int rad_tunnel_pwdecode(char *passwd, int * pwlen, const char *secret, const char *vector)
 {
 	uint8_t	buffer[AUTH_VECTOR_LEN + MAX_STRING_LEN + 3];
-	char	digest[AUTH_VECTOR_LEN];
-	char	r[AUTH_VECTOR_LEN];
+	unsigned char	digest[AUTH_VECTOR_LEN];
 	char    salt[2];
-	char	*s;
-	int	i, n, secretlen;
-	int	rlen;
+	int	i, n, ntimes, secretlen;
+	int 	len;
+	
+	len = *pwlen;
 
-	if(pwlen < 2) {
-	  return 0;
+	if(len < 3) {
+	  return len;
 	}
 	salt[0] = passwd[0];
 	salt[1] = passwd[1];
 
-	passwd = passwd + 2;
-        pwlen = pwlen - 2;
-
+	passwd += 2;
+	len -= 2;
+	
 	/*
 	 *	Use the secret to setup the decryption digest
 	 */
 	secretlen = strlen(secret);
 	memcpy(buffer, secret, secretlen);
-	memcpy(buffer + secretlen, vector, AUTH_VECTOR_LEN);
-	memcpy(buffer + secretlen + AUTH_VECTOR_LEN, salt, 2);
-	librad_md5_calc((u_char *)digest, buffer, secretlen + AUTH_VECTOR_LEN + 2);
 
-	/*
-	 *	Now we can decode the password *in place*
-	 */
-	memcpy(r, passwd, AUTH_PASS_LEN);
-	for (i = 0; i < AUTH_PASS_LEN && i < pwlen; i++)
-		passwd[i] ^= digest[i];
-
-	if (pwlen <= AUTH_PASS_LEN) {
-		passwd[pwlen+1] = '\0';
-		return pwlen;
-	}
-
-	/*
-	 *	Length > AUTH_PASS_LEN, so we need to use the extended
-	 *	algorithm.
-	 */
-	rlen = ((pwlen - 1) / AUTH_PASS_LEN) * AUTH_PASS_LEN;
-
-	for (n = rlen; n > 0; n -= AUTH_PASS_LEN ) { 
-		s = (n == AUTH_PASS_LEN) ? r : (passwd + n - AUTH_PASS_LEN);
-		memcpy(buffer + secretlen, s, AUTH_PASS_LEN);
-		librad_md5_calc((u_char *)digest, buffer, secretlen + AUTH_PASS_LEN);
-		for (i = 0; i < AUTH_PASS_LEN && (i + n) < pwlen; i++)
+	ntimes = (len-1)/AUTH_PASS_LEN;
+	do {
+		if(!ntimes){
+			memcpy(buffer + secretlen, vector, AUTH_VECTOR_LEN);
+			memcpy(buffer + secretlen + AUTH_VECTOR_LEN, salt, 2);
+			librad_md5_calc(digest, buffer, secretlen + AUTH_VECTOR_LEN + 2);
+		}
+		else {
+			memcpy(buffer + secretlen, passwd - AUTH_PASS_LEN, AUTH_PASS_LEN);
+			librad_md5_calc(digest, buffer, secretlen + AUTH_PASS_LEN);
+		}
+		for ( i = 0, n = ntimes * AUTH_PASS_LEN; i < AUTH_PASS_LEN && (i + n) < len; i++)
 			passwd[i + n] ^= digest[i];
-	}
-	passwd[pwlen] = '\0';
+	} while(ntimes--);
+	passwd[len] = '\0';
 
-	return pwlen;
+	if (*(unsigned char*)passwd > len) {
+		/* Pasword is broken, original password should be longer */
+		*pwlen = 2;
+		passwd[0]=passwd[1]=0;
+		return 0;
+	}
+	return *pwlen = *passwd + 2; /* restore original length */
 }
 
 /*
@@ -1467,7 +1465,7 @@ int rad_chap_encode(RADIUS_PACKET *packet, char *output, int id, VALUE_PAIR *pas
 
 	/*
 	 *	Note that the password VP can be EITHER
-	 *	a Password attribute (from a check-item list),
+	 *	a User-Password attribute (from a check-item list),
 	 *	or a CHAP-Password attribute (the client asking
 	 *	the library to encode it).
 	 */
