@@ -1,7 +1,7 @@
 /*
  * auth.c	User authentication.
  *
- * Version:	$Id: auth.c,v 1.95 2001/10/30 17:22:04 aland Exp $
+ * Version:	$Id: auth.c,v 1.101 2002/03/06 17:49:36 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  */
 
-static const char rcsid[] = "$Id: auth.c,v 1.95 2001/10/30 17:22:04 aland Exp $";
+static const char rcsid[] = "$Id: auth.c,v 1.101 2002/03/06 17:49:36 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -144,7 +144,7 @@ static int rad_authlog(const char *msg, REQUEST *request, int goodpass) {
 	 */
 	if (mainconfig.log_auth_badpass || mainconfig.log_auth_goodpass) {
 		if (!request->password) {
-			strcpy(clean_password, "<no Password attribute>");
+			strcpy(clean_password, "<no User-Password attribute>");
 		} else if (request->password->attribute == PW_CHAP_PASSWORD) {
 			strcpy(clean_password, "<CHAP-Password>");
 		} else {
@@ -260,29 +260,16 @@ int rad_check_password(REQUEST *request)
 	else
 		password_pair = pairfind(request->config_items, PW_PASSWORD);
 
-	/*
-	 *	For backward compatibility, we check the
-	 *	password to see if it is the magic value
-	 *	UNIX or PAM if auth_type was not set.
-	 *
-	 *	This allows the admin to just put a 'Password' in, and
-	 *	have it work.
-	 */
 	if (auth_type < 0) {
 		if (password_pair) {
-			if (!strcmp((char *)password_pair->strvalue, "UNIX"))
-				auth_type = PW_AUTHTYPE_SYSTEM;
-			else if (!strcmp((char *)password_pair->strvalue,"PAM"))
-				auth_type = PW_AUTHTYPE_PAM;
-			else
-				auth_type = PW_AUTHTYPE_LOCAL;
+			auth_type = PW_AUTHTYPE_LOCAL;
 		} else {
 			/*
-			 *	The admin hasn't told us how to
-			 *	authenticate the user, so we reject them!
-			 *
-			 *	This is fail-safe.
-			 */
+		 	*	The admin hasn't told us how to
+		 	*	authenticate the user, so we reject them!
+		 	*
+		 	*	This is fail-safe.
+		 	*/
 			DEBUG2("auth: No Auth-Type configuration for the request, rejecting the user");
 			return -2;
 		}
@@ -297,14 +284,15 @@ int rad_check_password(REQUEST *request)
 			 */
 			auth_item = request->password;
 			if (auth_item == NULL) {
-				DEBUG2("auth: No Password or CHAP-Password attribute in the request");
+				DEBUG2("auth: No User-Password or CHAP-Password attribute in the request");
 				return -1;
 			}
 
 			DEBUG2("auth: type Crypt");
 			if (password_pair == NULL) {
 				DEBUG2("No Crypt-Password configured for the user");
-				rad_authlog("No Crypt-Password configured for the user", request, 0);
+				rad_authlog("Login incorrect "
+					"(No Crypt-Password configured for the user)", request, 0);
 				return -1;
 			}
 					
@@ -323,7 +311,7 @@ int rad_check_password(REQUEST *request)
 			 */
 			auth_item = request->password;
 			if (auth_item == NULL) {
-				DEBUG2("auth: No Password or CHAP-Password attribute in the request");
+				DEBUG2("auth: No User-Password or CHAP-Password attribute in the request");
 				return -1;
 			}
 
@@ -331,8 +319,9 @@ int rad_check_password(REQUEST *request)
 			 *	Plain text password.
 			 */
 			if (password_pair == NULL) {
-				DEBUG2("auth: No Password configured for the user");
-				rad_authlog("No Password configured for the user", request, 0);
+				DEBUG2("auth: No password configured for the user");
+				rad_authlog("Login incorrect "
+					"(No password configured for the user)", request, 0);
 				return -1;
 			}
 
@@ -344,12 +333,13 @@ int rad_check_password(REQUEST *request)
 					   (char *)auth_item->strvalue) != 0) {
 					return -1;
 				}
-				DEBUG2("auth: user supplied Password matches local Password");
+				DEBUG2("auth: user supplied User-Password matches local User-Password");
 				break;
 
 			} else if (auth_item->attribute != PW_CHAP_PASSWORD) {
-				DEBUG2("The user did not supply a Password or a CHAP-Password attribute");
-				rad_authlog("The user did not supply a Password or a CHAP-Password attribute", request, 0);
+				DEBUG2("The user did not supply a User-Password or a CHAP-Password attribute");
+				rad_authlog("Login incorrect "
+					"(no User-Password or CHAP-Password attribute)", request, 0);
 				return -1;
 			}
 
@@ -362,7 +352,7 @@ int rad_check_password(REQUEST *request)
 			if (memcmp(string + 1, auth_item->strvalue + 1,
 					CHAP_VALUE_LENGTH) != 0)
 				result = -1;
-			DEBUG2("auth: user supplied CHAP-Password matches local Password");
+			DEBUG2("auth: user supplied CHAP-Password matches local User-Password");
 			break;
 		default:
 			DEBUG2("auth: type \"%s\"",
@@ -416,6 +406,7 @@ int rad_authenticate(REQUEST *request)
 	VALUE_PAIR	*reply_item;
 	VALUE_PAIR	*auth_item;
 	VALUE_PAIR	*tmp = NULL;
+	VALUE_PAIR	*autz_type_item = NULL;
 	int		result, r;
 	char		umsg[MAX_STRING_LEN + 1];
 	const char	*user_msg = NULL;
@@ -424,6 +415,8 @@ int rad_authenticate(REQUEST *request)
 	int		exec_wait;
 	int		seen_callback_id;
 	char		buf[1024], logstr[1024];
+	char		autz_retry = 0;
+	int		autz_type = 0;
 
 	password = "";
 
@@ -480,7 +473,7 @@ int rad_authenticate(REQUEST *request)
 		 */
 		if ((request->proxy_reply->code != PW_AUTHENTICATION_ACK) &&
 		    (request->proxy_reply->code != PW_ACCESS_CHALLENGE)) {
-			rad_authlog("Home server says invalid user", request, 0);
+			rad_authlog("Login incorrect (Home Server says so)", request, 0);
 			request->reply->code = PW_AUTHENTICATION_REJECT;
 			return RLM_MODULE_REJECT;
 		}
@@ -521,7 +514,8 @@ int rad_authenticate(REQUEST *request)
 	/*
 	 *	Get the user's authorization information from the database
 	 */
-	r = module_authorize(request);
+autz_redo:
+	r = module_authorize(autz_type, request);
 	if (r != RLM_MODULE_NOTFOUND &&
 			r != RLM_MODULE_NOOP &&
 			r != RLM_MODULE_OK &&
@@ -540,12 +534,22 @@ int rad_authenticate(REQUEST *request)
 			}
 			request->reply->code = PW_AUTHENTICATION_REJECT;
 		}
+
 		/*
 		 *	Hope that the module returning REJECT is smart
 		 *	enough to do pairfre(&request->reply->vps)...
 		 */
 		return r;
 	}
+	if (!autz_retry){
+		autz_type_item = pairfind(request->config_items, PW_AUTZTYPE);
+		if (autz_type_item){
+			autz_type = autz_type_item->lvalue;
+			autz_retry = 1;
+			goto autz_redo;
+		}
+	}
+
 
 	/*
 	 *	If we haven't already proxied the packet, then check
@@ -648,9 +652,9 @@ int rad_authenticate(REQUEST *request)
 				request->reply->code = PW_AUTHENTICATION_REJECT;
 
 				/*
-		 		*	They're trying to log in too many times.
-		 		*	Remove ALL reply attributes.
-		 		*/
+				 *	They're trying to log in too many times.
+				 *	Remove ALL reply attributes.
+				 */
 				pairfree(&request->reply->vps);
 				tmp = pairmake("Reply-Message", user_msg, T_OP_SET);
 				request->reply->vps = tmp;
@@ -666,7 +670,7 @@ int rad_authenticate(REQUEST *request)
 	}
 
 	if (result >= 0 &&
-			(check_item = pairfind(request->config_items, PW_LOGIN_TIME)) != NULL) {
+	    (check_item = pairfind(request->config_items, PW_LOGIN_TIME)) != NULL) {
 
 		/*
 		 *	Authentication is OK. Now see if this
@@ -739,7 +743,7 @@ int rad_authenticate(REQUEST *request)
 	 *	vp->addport is set.
 	 */
 	if (((tmp = pairfind(request->reply->vps, 
-			    PW_FRAMED_IP_ADDRESS)) != NULL) &&
+			     PW_FRAMED_IP_ADDRESS)) != NULL) &&
 	    (tmp->flags.addport != 0)) {
 		VALUE_PAIR *vpPortId;
 		
@@ -750,6 +754,9 @@ int rad_authenticate(REQUEST *request)
 					 PW_NAS_PORT_ID)) != NULL) {
 		  tmp->lvalue = htonl(ntohl(tmp->lvalue) + vpPortId->lvalue);
 		  tmp->flags.addport = 0;
+		} else {
+			DEBUG2("WARNING: No NAS-Port attribute in request.  CANNOT return a Framed-IP-Address + NAS-Port.\n");
+			pairdelete(&request->reply->vps, PW_FRAMED_IP_ADDRESS);
 		}
 	}
 
