@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: conffile.c,v 1.78 2001/11/08 16:58:56 aland Exp $
+ * Version:	$Id: conffile.c,v 1.80 2002/02/26 21:46:17 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
 #include "modules.h"
 
 static const char rcsid[] =
-"$Id: conffile.c,v 1.78 2001/11/08 16:58:56 aland Exp $";
+"$Id: conffile.c,v 1.80 2002/02/26 21:46:17 aland Exp $";
 
 #define xstrdup strdup
 
@@ -750,7 +750,6 @@ static CONF_PARSER directory_config[] = {
 	{ NULL, -1, 0, NULL, NULL }
 };
 
-
 int read_radius_conf_file(void)
 {
 	char buffer[256];
@@ -906,7 +905,7 @@ static int generate_realms(const char *filename)
 				c->acct_ipaddr = htonl(INADDR_NONE);
 				c->acct_port = acct_port;
 			} else {
-				c->acct_ipaddr = ip_getaddr(authhost);
+				c->acct_ipaddr = ip_getaddr(accthost);
 			}
 		}
 
@@ -929,20 +928,28 @@ static int generate_realms(const char *filename)
 		strcpy(c->realm, cs->name2);
 		strcpy(c->server, authhost);	
 
-		if ((s = cf_section_value_find(cs, "secret")) == NULL ) {
-			radlog(L_ERR, "%s[%d]: No shared secret supplied for realm: %s",
-					filename, cs->item.lineno, cs->name2);
-			return -1;
+		/*
+		 *	If one or the other of authentication/accounting
+		 *	servers is set to LOCALHOST, then don't require
+		 *	a shared secret.
+		 */
+		if ((c->ipaddr != htonl(INADDR_NONE)) ||
+		    (c->acct_ipaddr != htonl(INADDR_NONE))) {
+			if ((s = cf_section_value_find(cs, "secret")) == NULL ) {
+				radlog(L_ERR, "%s[%d]: No shared secret supplied for realm: %s",
+				       filename, cs->item.lineno, cs->name2);
+				return -1;
+			}
+			
+			if (strlen(s) >= sizeof(c->secret)) {
+				radlog(L_ERR, "%s[%d]: Secret of length %d is greater than the allowed maximum of %d.",
+				       filename, cs->item.lineno,
+				       strlen(s), sizeof(c->secret) - 1);
+				return -1;
+			}
+			strNcpy((char *)c->secret, s, sizeof(c->secret));
 		}
 
-		if (strlen(s) >= sizeof(c->secret)) {
-			radlog(L_ERR, "%s[%d]: Secret of length %d is greater than the allowed maximum of %d.",
-					filename, cs->item.lineno,
-					strlen(s), sizeof(c->secret) - 1);
-			return -1;
-		}
-		strNcpy((char *)c->secret, s, sizeof(c->secret));
-
 		c->striprealm = 1;
 		
 		if ((cf_section_value_find(cs, "nostrip")) != NULL)
