@@ -1,7 +1,7 @@
 /*
  * files.c	Read config files into memory.
  *
- * Version:     $Id: files.c,v 1.58 2001/11/05 15:18:51 aland Exp $
+ * Version:     $Id: files.c,v 1.62 2002/03/06 17:55:39 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: files.c,v 1.58 2001/11/05 15:18:51 aland Exp $";
+static const char rcsid[] = "$Id: files.c,v 1.62 2002/03/06 17:55:39 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -65,10 +65,10 @@ void pairlist_free(PAIR_LIST **pl)
 
 /*
  *	Fixup a check line.
- *	If Password or Crypt-Password is set, but there is no
+ *	If User-Password or Crypt-Password is set, but there is no
  *	Auth-Type, add one (kludge!).
  */
-static void auth_type_fixup(VALUE_PAIR *check)
+static void auth_type_fixup(VALUE_PAIR **check)
 {
 	VALUE_PAIR *vp;
 	VALUE_PAIR *c = NULL;
@@ -78,7 +78,7 @@ static void auth_type_fixup(VALUE_PAIR *check)
 	 *	See if a password is present. Return right away
 	 *	if we see Auth-Type.
 	 */
-	for (vp = check; vp; vp = vp->next) {
+	for (vp = *check; vp; vp = vp->next) {
 		if (vp->attribute == PW_AUTHTYPE)
 			return;
 		if (vp->attribute == PW_PASSWORD) {
@@ -105,14 +105,10 @@ static void auth_type_fixup(VALUE_PAIR *check)
 	vp->lvalue = n;
 	vp->operator = T_OP_ADD;
 
-#if 0
-	vp->next = c->next;
-	c->next = vp;
-#endif
-	vp->next = check;
-	check = vp;
+	vp->next = *check;
+	*check = vp;
 
-	for(vp = check; vp; vp = vp->next) {
+	for(vp = *check; vp; vp = vp->next) {
 		DEBUG2("  auth_type_fixup: %s [%d]", vp->name, vp->attribute);
 	}
 
@@ -284,7 +280,7 @@ parse_again:
 				 */
 				t = rad_malloc(sizeof(PAIR_LIST));
 
-				auth_type_fixup(check_tmp);
+				auth_type_fixup(&check_tmp);
 				memset(t, 0, sizeof(*t));
 				t->name = strdup(entry);
 				t->check = check_tmp;
@@ -508,6 +504,8 @@ void realm_disable(uint32_t ipaddr)
 		    (ipaddr == cl->acct_ipaddr)) {
 			cl->active = FALSE;
 			cl->wakeup = now + proxy_dead_time;
+			radlog(L_PROXY, "marking server %s for realm %s dead",
+				cl->server, cl->realm);
 		}
 }
 
@@ -519,6 +517,7 @@ REALM *realm_find(const char *realm)
 	REALM *cl;
 	REALM *default_realm = NULL;
 	time_t now;
+	int dead_match = 0;
 
 	now = time(NULL);
 
@@ -542,6 +541,17 @@ REALM *realm_find(const char *realm)
 		 *	It's not alive, skip it.
 		 */
 		if (!cl->active) {
+
+			/*
+			 *	We've been asked to NOT fall through
+			 *	to the DEFAULT realm if there are
+			 *	exact matches for this realm which are
+			 *	dead.
+			 */
+			if ((!proxy_fallback) &&
+			    (strcmp(cl->realm, realm) == 0)) {
+				dead_match = 1;
+			}
 			continue;
 		}
 
@@ -562,8 +572,21 @@ REALM *realm_find(const char *realm)
 	} /* loop over all realms */
 
 	/*
-	 *	Didn't find anything that matched exactly, return
-	 *	the default.
+	 *	There WAS one or more matches which were marked dead,
+	 *	AND there were NO live matches, AND we've been asked
+	 *	to NOT fall through to the DEFAULT realm.  Therefore,
+	 *	we return NULL, which means "no match found".
+	 */
+	if (!proxy_fallback && dead_match) {
+		return NULL;
+	}
+
+	/*
+	 *	Didn't find anything that matched exactly, return the
+	 *	DEFAULT realm.  We also return the DEFAULT realm if
+	 *	all matching realms were marked dead, and we were
+	 *	asked to fall through to the DEFAULT realm in this
+	 *	case.
 	 */
 	return default_realm;
 }
