@@ -1,7 +1,7 @@
 /*
  * log.c	Logging module.
  *
- * Version:	$Id: log.c,v 1.23 2001/10/31 15:29:18 aland Exp $
+ * Version:	$Id: log.c,v 1.26 2002/02/19 18:29:58 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2001  Chad Miller <cmiller@surfsouth.com>
  */
 
-static const char rcsid[] = "$Id: log.c,v 1.23 2001/10/31 15:29:18 aland Exp $";
+static const char rcsid[] = "$Id: log.c,v 1.26 2002/02/19 18:29:58 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -74,7 +74,7 @@ static int r_mkdir(const char *part) {
 }
 		
 
-int radlogdir_iswritable(char *effectiveuser) {
+int radlogdir_iswritable(const char *effectiveuser) {
 	struct passwd *pwent;
 
 	if (radlog_dir[0] != '/')
@@ -133,7 +133,7 @@ static int do_log(int lvl, const char *fmt, va_list ap)
 		} else if (radlog_dest == RADLOG_STDERR) {
 			msgfd = stderr;
 
-		} else {
+		} else if (radlog_dest != RADLOG_SYSLOG) {
 
 			sprintf(buffer, "%.1000s/%.1000s", radlog_dir, RADIUS_LOG);
 			if ((msgfd = fopen(buffer, "a")) == NULL) {
@@ -153,7 +153,7 @@ static int do_log(int lvl, const char *fmt, va_list ap)
 	if (radlog_dest == RADLOG_SYSLOG)
 		*buffer = '\0';
 	else {
-		strcpy(buffer, ctime(&timeval));
+		ctime_r(&timeval, buffer);
 
 		switch(lvl & ~L_CONS) {
 			case L_DBG:
