@@ -1,7 +1,7 @@
 /*
  * modcall.c
  *
- * Version:	$Id: modcall.c,v 1.10 2001/08/17 19:45:25 aland Exp $
+ * Version:	$Id: modcall.c,v 1.11 2002/01/14 11:22:49 3APA3A Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ static modcallable *do_compile_modgroup(int, CONF_SECTION *, const char *,
 struct modcallable {
 	struct modcallable *next;
 	int actions[RLM_MODULE_NUMCODES];
-	char *name;
+	const char *name;
 	enum { MOD_SINGLE, MOD_GROUP } type;
 };
 
