@@ -2,7 +2,7 @@
  * nas.c	Functions to do with a NASLIST. This is here because
  *		radzap needs it as well.
  *
- * Version:     $Id: nas.c,v 1.16 2001/03/05 03:42:11 cmiller Exp $
+ * Version:     $Id: nas.c,v 1.17 2002/03/02 05:50:43 ramoore Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: nas.c,v 1.16 2001/03/05 03:42:11 cmiller Exp $";
+static const char rcsid[] = "$Id: nas.c,v 1.17 2002/03/02 05:50:43 ramoore Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -200,7 +200,7 @@ const char *nas_name(uint32_t ipaddr)
 			return nas->longname;
 	}
 
-	return client_name(ipaddr);
+	return "UNKNOWN-NAS";
 }
 
 /*
@@ -217,6 +217,29 @@ const char *nas_name2(RADIUS_PACKET *packet)
 			return nas->longname;
 	}
 
-	return client_name(packet->src_ipaddr);
+	return "UNKNOWN-NAS";
 }
 
+/*
+ *	Find the name of a nas (prefer short name) based on ipaddr, 
+ *	store in passed buffer.  If NAS is unknown, return dotted quad.
+ */
+char * nas_name3(char *buf, size_t buflen, uint32_t ipaddr)
+{
+	NAS *nas;
+
+	if ((nas = nas_find(ipaddr)) != NULL) {
+		if (nas->shortname[0]) {
+			strNcpy(buf, (char *)nas->shortname, buflen);
+			return buf;
+		}
+		else {
+			strNcpy(buf, (char *)nas->longname, buflen);
+			return buf;
+		}
+	}
+	ip_ntoa(buf, ipaddr);
+	return buf;
+}
+
+
