@@ -1,7 +1,7 @@
 /*
  * radiusd.c	Main loop of the radius server.
  *
- * Version:	$Id: radiusd.c,v 1.231 2001/11/20 17:13:22 cmiller Exp $
+ * Version:	$Id: radiusd.c,v 1.242 2002/03/08 16:35:26 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 
 /* don't look here for the version, run radiusd -v or look in version.c */
 static const char rcsid[] =
-"$Id: radiusd.c,v 1.231 2001/11/20 17:13:22 cmiller Exp $";
+"$Id: radiusd.c,v 1.242 2002/03/08 16:35:26 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -110,6 +110,7 @@ int proxy_retry_delay = RETRY_DELAY;
 int proxy_retry_count = RETRY_COUNT;
 int proxy_dead_time = DEAD_TIME;
 int proxy_synchronous = FALSE;
+int proxy_fallback = FALSE;
 int need_reload = FALSE;
 struct main_config_t mainconfig;
 const char *radiusd_version = "FreeRADIUS Version " RADIUSD_VERSION ", for host " HOSTINFO ", built on " __DATE__ " at " __TIME__;
@@ -129,6 +130,7 @@ static int max_request_time = MAX_REQUEST_TIME;
 static int kill_unresponsive_children = FALSE;
 static int cleanup_delay = CLEANUP_DELAY;
 static int max_requests = MAX_REQUESTS;
+static int reject_delay = 0;
 static int dont_fork = FALSE;
 static const char *pid_file = NULL;
 static uid_t server_uid;
@@ -165,10 +167,20 @@ static CONF_PARSER proxy_config[] = {
 	{ "retry_delay",  PW_TYPE_INTEGER, 0, &proxy_retry_delay, Stringify(RETRY_DELAY) },
 	{ "retry_count",  PW_TYPE_INTEGER, 0, &proxy_retry_count, Stringify(RETRY_COUNT) },
 	{ "synchronous",  PW_TYPE_BOOLEAN, 0, &proxy_synchronous, "no" },
+	{ "default_fallback", PW_TYPE_BOOLEAN, 0, &proxy_fallback, "no" },
 	{ "dead_time",    PW_TYPE_INTEGER, 0, &proxy_dead_time, Stringify(DEAD_TIME) },
 	{ NULL, -1, 0, NULL, NULL }
 };
 
+/*
+ *  Security configuration for the server.
+ */
+static CONF_PARSER security_config[] = {
+	{ "max_attributes",  PW_TYPE_INTEGER, 0, &librad_max_attributes, Stringify(0) },
+	{ "reject_delay",  PW_TYPE_INTEGER, 0, &reject_delay, Stringify(0) },
+	{ NULL, -1, 0, NULL, NULL }
+};
+
 /*
  *  A mapping of configuration file names to internal variables
  */
@@ -194,11 +206,11 @@ static CONF_PARSER server_config[] = {
 	{ "nospace_pass", PW_TYPE_STRING_PTR, 0, &mainconfig.do_nospace_pass, "no" },
 	{ "proxy_requests", PW_TYPE_BOOLEAN, 0, &proxy_requests, "yes" },
 	{ "proxy", PW_TYPE_SUBSECTION, 0, proxy_config, NULL },
+	{ "security", PW_TYPE_SUBSECTION, 0, security_config, NULL },
 	{ "debug_level", PW_TYPE_INTEGER, 0, &debug_level, "0"},
 	{ NULL, -1, 0, NULL, NULL }
 };
 
-
 static int switch_users() {
 	
 	/*
@@ -376,6 +388,13 @@ static int reread_config(int reload)
 		switch_users();  /* Don't do this yet, if we're just starting up. */
 	}
 
+	/*
+	 *	Sanity check some things...
+	 */
+	if (reject_delay > cleanup_delay) {
+		reject_delay = cleanup_delay;
+	}
+
 	return 0;
 }
 
@@ -506,11 +525,10 @@ int main(int argc, char *argv[])
 	fd_set readfds;
 	int result;
 	int argval;
-	int t;
 	int pid;
 	int i;
 	int fd = 0;
-	int devnull;
+	int max_fd;
 	int status;
 	int radius_port = 0;
 	struct servent *svp;
@@ -522,14 +540,6 @@ int main(int argc, char *argv[])
 	set_auth_parameters(argc,argv);
 #endif
 
-	/*
-	 *  Open /dev/null, and make sure filedescriptors
-	 *  0, 1 and 2 are connected to something.
-	 */
-	devnull = 0;
-	while (devnull >= 0 && devnull < 3)
-		devnull = open("/dev/null", O_RDWR);
-
 	if ((progname = strrchr(argv[0], '/')) == NULL)
 		progname = argv[0];
 	else
@@ -550,11 +560,6 @@ int main(int argc, char *argv[])
 	 */
 	signal(SIGCHLD, sig_cleanup);
 
-	/*  Close unused file descriptors.  */
-	for (t = 32; t >= 3; t--)
-		if(t!=devnull) 
-			close(t);
-
 	/*  Process the options.  */
 	while ((argval = getopt(argc, argv, "Aa:bcd:fg:hi:l:p:sSvxXyz")) != EOF) {
 
@@ -565,7 +570,7 @@ int main(int argc, char *argv[])
 				break;
 
 			case 'a':
-				if (radacct_dir) free(radacct_dir);
+				if (radacct_dir) xfree(radacct_dir);
 				radacct_dir = strdup(optarg);
 				break;
 			
@@ -574,7 +579,7 @@ int main(int argc, char *argv[])
 				break;
 
 			case 'd':
-				if (radius_dir) free(radius_dir);
+				if (radius_dir) xfree(radius_dir);
 				radius_dir = strdup(optarg);
 				break;
 			
@@ -831,22 +836,6 @@ int main(int argc, char *argv[])
 	radius_snmp_init();
 #endif
 
-#if 0
-	/*
-	 *  Connect 0, 1 and 2 to /dev/null.
-	 */
-	if (!debug_flag && devnull >= 0) {
-		dup2(devnull, 0);
-		if (strcmp(radlog_dir, "stdout") != 0) {
-			dup2(devnull, 1);
-		}
-		if (strcmp(radlog_dir, "stderr") != 0) {
-			dup2(devnull, 2);
-		}
-		if (devnull > 2) close(devnull);
-	}
-#endif
-
 	/*
 	 *  Disconnect from session
 	 */
@@ -893,6 +882,25 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	/*
+	 *	If we're running as a daemon, close the default file
+	 *	descriptors, AFTER forking.
+	 */
+	if (debug_flag == FALSE) {
+		int devnull;
+		
+		devnull = open("/dev/null", O_RDWR);
+		if (devnull < 0) {
+			radlog(L_ERR|L_CONS, "Failed opening /dev/null: %s\n",
+			       strerror(errno));
+			exit(1);
+		}
+		dup2(devnull, STDIN_FILENO);
+		dup2(devnull, STDOUT_FILENO);
+		dup2(devnull, STDERR_FILENO);
+		close(devnull);
+	}
+
 #if HAVE_PTHREAD_H
 	/*
 	 *  If we're spawning children, set up the thread pool.
@@ -925,14 +933,6 @@ int main(int argc, char *argv[])
 
 	radlog(L_INFO, "Ready to process requests.");
 
-
-	/* if we're running as a daemon, close other file descriptors. */
-	if (debug_flag == FALSE) {
-		close(STDIN_FILENO);
-		close(STDOUT_FILENO);
-		close(STDERR_FILENO);
-	}
-
 	/*
 	 *  Receive user requests
 	 */
@@ -946,18 +946,27 @@ int main(int argc, char *argv[])
 		}
 
 		FD_ZERO(&readfds);
-		if (authfd >= 0)
+		max_fd = 0;
+		if (authfd >= 0) {
 			FD_SET(authfd, &readfds);
-		if (acctfd >= 0)
+			if (authfd > max_fd) max_fd = authfd;
+		}
+		if (acctfd >= 0) {
 			FD_SET(acctfd, &readfds);
-		if (proxyfd >= 0)
+			if (acctfd > max_fd) max_fd = acctfd;
+		}
+		if (proxyfd >= 0) {
 			FD_SET(proxyfd, &readfds);
+			if (proxyfd > max_fd) max_fd = proxyfd;
+		}
 #ifdef WITH_SNMP
-		if (rad_snmp.smux_fd >= 0)
+		if (rad_snmp.smux_fd >= 0) {
 			FD_SET(rad_snmp.smux_fd, &readfds);
+			if (rad_snmp.smux_fd > max_fd) max_fd = rad_snmp.smux_fd;
+		}
 #endif
 
-		status = select(32, &readfds, NULL, NULL, tv);
+		status = select(max_fd + 1, &readfds, NULL, NULL, tv);
 		if (status == -1) {
 			/*
 			 *  On interrupts, we clean up the
@@ -1209,6 +1218,17 @@ int rad_process(REQUEST *request, int dospawn)
 	
 	rad_assert(request->magic == REQUEST_MAGIC);
 
+	/*
+	 *  This next assertion catches a race condition in the
+	 *  server.  If it core dumps here, then it means that
+	 *  the code WOULD HAVE core dumped elsewhere, but in
+	 *  some random, unpredictable location.
+	 *
+	 *  Having the assert here means that we can catch the problem
+	 *  in a well-known manner, until such time as we fix it.
+	 */
+	rad_assert(request->child_pid == NO_SUCH_CHILD_PID);
+
 	/*
 	 *  The request passes many of our sanity checks.  From
 	 *  here on in, if anything goes wrong, we send a reject
@@ -1294,8 +1314,20 @@ static void rad_reject(REQUEST *request)
 	/*
 	 *  If a reply exists, send it.
 	 */
-	if (request->reply->code != 0) 
-		rad_send(request->reply, request->packet, request->secret);
+	if (request->reply->code != 0) {
+		/*
+		 *	If we're not delaying authentication rejects,
+		 *	then send the response immediately.  Otherwise,
+		 *	mark the request as delayed, and do NOT send a
+		 *	response.
+		 */
+		if (reject_delay == 0) {
+			rad_send(request->reply, request->packet,
+				 request->secret);
+		} else {
+			request->options |= RAD_REQUEST_OPTION_DELAYED_REJECT;
+		}
+	}
 }
 
 /*
@@ -1524,7 +1556,24 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 	 */
 	if (proxy_requests) {
 		if (request->proxy == NULL) {
+			/*
+			 *  Try to proxy this request.  Returns:
+			 *  -1: error, drop the request
+			 *   0: OK, but don't proxy it.
+			 *   1: OK, it's been proxied, don't do any more here.
+			 *   2: OK, it's been proxied to one or more servers.
+			 */
 			proxy_sent = proxy_send(request);
+
+			/*
+			 *  There was an error trying to proxy the request.
+			 *  Drop it on the floor.
+			 */
+			if (proxy_sent < 0) {
+				DEBUG2("Error trying to proxy request %d: Rejecting it", request->number);
+				rad_reject(request);
+				goto finished_request;
+			}
 			
 			/*
 			 *  sent==1 means it's been proxied.  The child
@@ -1532,7 +1581,6 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 			 *  is NOT finished!
 			 */
 			if (proxy_sent == 1) {
-				finished = request->finished;
 				goto postpone_request;
 			}
 		}
@@ -1569,7 +1617,23 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 		if (vp != NULL) 
 			pairadd(&(request->reply->vps), vp);
 
-		rad_send(request->reply, request->packet, request->secret);
+		/*
+		 *  If the request isn't an authentication reject, OR
+		 *  it's a reject, but the reject_delay is zero, then
+		 *  send it immediately.
+		 *
+		 *  Otherwise, delay the authentication reject to shut
+		 *  up DoS attacks.
+		 */
+		if ((request->reply->code != PW_AUTHENTICATION_REJECT) ||
+		    (reject_delay == 0)) {
+			rad_send(request->reply, request->packet,
+				 request->secret);
+		} else {
+			DEBUG2("Delaying request %d for %d seconds",
+			       request->number, reject_delay);
+			request->options |= RAD_REQUEST_OPTION_DELAYED_REJECT;
+		}
 	}
 
 	/*
@@ -1603,7 +1667,6 @@ finished_request:
 	 *  by a thread.
 	 */
 	if (proxy_sent) {
-		finished = request->finished;
 		goto postpone_request;
 	}
 
@@ -1801,7 +1864,7 @@ static struct timeval *rad_clean_list(time_t now)
 }
 
 /*
- *  Walk through the request list, cleaning up complete child
+ *  Walk through the request list, cleaning up completed child
  *  requests, and verifing that there is only one process
  *  responding to each request (duplicate requests are filtered
  *  out).
@@ -1915,6 +1978,18 @@ static REQUEST *rad_check_list(REQUEST *request)
 			if (last_request == curreq) {
 				last_request = rl_next(last_request);
 			}
+
+			/*
+			 *	If we're keeping a delayed reject, and we
+			 *	get a new request, then we send the reject
+			 *	before deleting it.
+			 */
+			if ((curreq->options & RAD_REQUEST_OPTION_DELAYED_REJECT) != 0) {
+			  curreq->options &= ~RAD_REQUEST_OPTION_DELAYED_REJECT;
+			  rad_send(curreq->reply, curreq->packet,
+				   curreq->secret);
+			}
+
 			rl_delete(curreq);
 
 			/*
@@ -2070,7 +2145,7 @@ void sig_cleanup(int sig)
 	 *	Wait for the child, without hanging.
 	 */
 	for (;;) {
-		pid = waitpid((pid_t)-1, &status, WNOHANG);
+		pid = (child_pid_t) waitpid((pid_t)-1, &status, WNOHANG);
 		if ((int)pid <= 0)
 			return;
 
@@ -2082,7 +2157,7 @@ void sig_cleanup(int sig)
 		if (debug_flag && (WIFSIGNALED(status))) {
 			radlog(L_ERR|L_CONS, "MASTER: Child PID %d failed to catch "
 					"signal %d: killing all active servers.\n",
-					pid, WTERMSIG(status));
+					(int)pid, WTERMSIG(status));
 			kill(0, SIGTERM);
 			exit(1);
 		}
@@ -2286,6 +2361,27 @@ static int refresh_request(REQUEST *request, void *data)
 
 	rad_assert(request->magic == REQUEST_MAGIC);
 
+	/*
+	 *  If the request is marked as a delayed reject, AND it's
+	 *  time to send the reject, then do so now.
+	 */
+	if (request->finished &&
+	    ((request->options & RAD_REQUEST_OPTION_DELAYED_REJECT) != 0)) {
+		rad_assert(request->child_pid == NO_SUCH_CHILD_PID);
+
+		difference = info->now - request->timestamp;
+		if (difference >= (time_t) reject_delay) {
+
+			/*
+			 *  Clear the 'delayed reject' bit, so that we
+			 *  don't do this again.
+			 */
+			request->options &= ~RAD_REQUEST_OPTION_DELAYED_REJECT;
+			rad_send(request->reply, request->packet,
+				 request->secret);
+		}
+	}
+
 	/*
 	 *  If the request has finished processing,
 	 *  AND it's child has been cleaned up,
@@ -2299,9 +2395,9 @@ static int refresh_request(REQUEST *request, void *data)
 	 *  seriously wrong...
 	 */
 	if (request->finished &&
-	    (request->child_pid == NO_SUCH_CHILD_PID) &&
 	    ((request->timestamp + cleanup_delay <= info->now) ||
 	     (request->packet->code == PW_ACCOUNTING_REQUEST))) {
+		rad_assert(request->child_pid == NO_SUCH_CHILD_PID);
 		/*
 		 *  Request completed, delete it, and unlink it
 		 *  from the currently 'alive' list of requests.
@@ -2346,19 +2442,13 @@ static int refresh_request(REQUEST *request, void *data)
 			} /* else no proxy reply, quietly fail */
 		
 			/*
-			 *  Delete the request.
+			 *	Maybe we haven't killed it.  In that
+			 *	case, print a warning.
 			 */
-			rl_delete(request);
-			child_pid = NO_SUCH_CHILD_PID; /* mark it as deleted. */
-
-		/*
-		 *	Maybe we haven't killed it.  In that case, print
-		 *	a warning.
-		 */
 		} else if ((child_pid != NO_SUCH_CHILD_PID) &&
 			   ((request->options & RAD_REQUEST_OPTION_LOGGED_CHILD) == 0)) {
 			radlog(L_ERR, "WARNING: Unresponsive child (id %lu) for request %d",
-			       child_pid, number);
+			       (unsigned long)child_pid, number);
 
 			/*
 			 *  Set the option that we've sent a log message,
@@ -2367,6 +2457,23 @@ static int refresh_request(REQUEST *request, void *data)
 			 */
 			request->options |= RAD_REQUEST_OPTION_LOGGED_CHILD;
 		}
+
+		/*
+		 *  Send a reject message for the request, mark it
+		 *  finished, and forget about the child.
+		 */
+		rad_reject(request);
+		request->child_pid = NO_SUCH_CHILD_PID;
+		request->finished = TRUE;
+		return RL_WALK_CONTINUE;
+	} /* the request has been in the queue for too long */
+
+	/*
+	 *  If the request is still being processed, then due to the
+	 *  above check, it's still within it's time limit.  In that
+	 *  case, don't do anything.
+	 */
+	if (request->child_pid != NO_SUCH_CHILD_PID) {
 		return RL_WALK_CONTINUE;
 	}
 
@@ -2416,9 +2523,9 @@ static int refresh_request(REQUEST *request, void *data)
 	 *  the request as finished, and go to the next one.
 	 */
 	if (request->proxy_try_count == 0) {
-		request->finished = TRUE;
 		rad_reject(request);
 		realm_disable(request->proxy->dst_ipaddr);
+		request->finished = TRUE;
 		goto setup_timeout;
 	}
 
@@ -2477,7 +2584,17 @@ setup_timeout:
 	 */
 	if (request->finished) {
 		difference = (request->timestamp + cleanup_delay) - info->now;
-		
+
+		/*
+		 *  If the request is marked up to be rejected later,
+		 *  then wake up later.
+		 */
+		if ((request->options & RAD_REQUEST_OPTION_DELAYED_REJECT) != 0) {
+			if (difference >= (time_t) reject_delay) {
+				difference = (time_t) reject_delay;
+			}
+		}
+
 	} else if (request->proxy && !request->proxy_reply) {
 		/*
 		 *  The request is NOT finished, but there is an
