@@ -1,7 +1,7 @@
 /*
  * session.c	session management
  *
- * Version:	$Id: session.c,v 1.7 2001/08/15 20:45:33 aland Exp $
+ * Version:	$Id: session.c,v 1.10 2002/03/11 16:42:41 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@
 #include	"modules.h"
 
 /* End a session by faking a Stop packet to all accounting modules */
-int session_zap(uint32_t nasaddr, int port, const char *user,
+int session_zap(int sockfd, uint32_t nasaddr, int port, const char *user,
 		const char *sessionid, uint32_t cliaddr, char proto, time_t t)
 {
 	static unsigned char id = 0;
@@ -48,7 +48,7 @@ int session_zap(uint32_t nasaddr, int port, const char *user,
 	stoppkt = rad_malloc(sizeof *stoppkt);
 	memset(stoppkt, 0, sizeof stoppkt);
 	stoppkt->data = NULL;
-	stoppkt->sockfd = acctfd;
+	stoppkt->sockfd = sockfd;
 	stoppkt->code = PW_ACCOUNTING_REQUEST;
 	stoppkt->id = id++;
 	stoppkt->timestamp = t?t:time(0);
@@ -150,8 +150,12 @@ int rad_check_ts(uint32_t nasaddr, int portnum, const char *user,
 	 *	Find NAS type.
 	 */
 	if ((nas = nas_find(nasaddr)) == NULL) {
-		radlog(L_ERR, "Accounting: unknown NAS");
-		return -1;
+	/*	radlog(L_ERR, "Accounting: unknown NAS, so trusting radutmp");	*/
+		return 1;
+	}
+	if (nas->nastype == "other") {
+	/*	radlog(L_ERR, "Accounting: NAS type is 'other', so trusting radutmp");	*/
+		return 1;
 	}
 
 	/*
@@ -194,7 +198,7 @@ int rad_check_ts(uint32_t nasaddr, int portnum, const char *user,
 	/*
 	 *	Child - exec checklogin with the right parameters.
 	 */
-	for (n = 32; n >= 3; n--)
+	for (n = 256; n >= 3; n--)
 		close(n);
 
 	ip_ntoa(address, nasaddr);
