@@ -1,7 +1,7 @@
 /*
  * timestr.c	See if a string like 'Su2300-0700' matches (UUCP style).
  *
- * Version:	$Id: timestr.c,v 1.13 2001/06/30 00:00:55 cmiller Exp $
+ * Version:	$Id: timestr.c,v 1.14 2002/02/01 16:17:08 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: timestr.c,v 1.13 2001/06/30 00:00:55 cmiller Exp $";
+static const char rcsid[] = "$Id: timestr.c,v 1.14 2002/02/01 16:17:08 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -203,7 +203,7 @@ static int week_fill(char *bitmap, char *tm)
  */
 int timestr_match(char *tmstr, time_t t)
 {
-	struct tm *tm;
+	struct tm *tm, s_tm;
 	char bitmap[WEEKMIN / 8];
 	int now, tot, i;
 	int byte, bit;
@@ -213,7 +213,7 @@ int timestr_match(char *tmstr, time_t t)
 	char null[8];
 #endif
 
-	tm = localtime(&t);
+	tm = localtime_r(&t, &s_tm);
 	now = tm->tm_wday * DAYMIN + tm->tm_hour * 60 + tm->tm_min;
 	tot = 0;
 	memset(bitmap, 0, sizeof(bitmap));
