@@ -1,7 +1,7 @@
 /*
  * util.c	Various utility functions.
  *
- * Version:     $Id: util.c,v 1.28 2001/08/17 19:45:27 aland Exp $
+ * Version:     $Id: util.c,v 1.29 2002/02/20 21:35:36 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: util.c,v 1.28 2001/08/17 19:45:27 aland Exp $";
+static const char rcsid[] = "$Id: util.c,v 1.29 2002/02/20 21:35:36 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -106,8 +106,13 @@ void request_free(REQUEST **request_ptr)
 	if (request->config_items) 
 		pairfree(&request->config_items);
 
+	request->username = NULL;
+	request->password = NULL;
+
 #ifndef NDEBUG
 	request->magic = 0x01020304;	/* set the request to be nonsense */
+	strcpy(request->secret, "REQUEST-DELETED");
+	strcpy(request->proxysecret, "REQUEST-DELETED");
 #endif
 	free(request);
 
