@@ -2,7 +2,7 @@
  * xlat.c	Translate strings.  This is the first version of xlat 
  * 		incorporated to RADIUS
  *
- * Version:	$Id: xlat.c,v 1.34 2001/11/27 22:44:24 kkalev Exp $
+ * Version:	$Id: xlat.c,v 1.39 2002/02/25 16:02:23 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 static const char rcsid[] = 
-"$Id: xlat.c,v 1.34 2001/11/27 22:44:24 kkalev Exp $";
+"$Id: xlat.c,v 1.39 2002/02/25 16:02:23 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -35,7 +35,7 @@ static const char rcsid[] =
 
 #include	"radiusd.h"
 
-static struct xlat_cmp {
+struct xlat_cmp {
 	char module[MAX_STRING_LEN];
 	int length;
 	void *instance;
@@ -208,6 +208,13 @@ static void decode_attribute(const char **from, char **to, int freespace, int *o
 			q += valuepair2str(q,freespace,tmppair,tmpda->type, func);
 			found = 1;
 		}
+	} else if ((strncasecmp(attrname,"proxy-reply:",12) == 0) &&
+		   (request->proxy_reply != NULL)) {
+		if((tmpda = dict_attrbyname(&attrname[12])) && 
+				(tmppair = pairfind(request->proxy_reply->vps, tmpda->attr))) {
+			q += valuepair2str(q,freespace,tmppair,tmpda->type, func);
+			found = 1;
+		}
 	} else if ((c = find_xlat_func(attrname)) != NULL){
 		DEBUG("radius_xlat: Runing registered xlat function of module %s for string \'%s\'",
 				c->module, attrname+(c->length+1));
@@ -285,7 +292,7 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 	const char *p;
 	char *q;
 	VALUE_PAIR *tmp;
-	struct tm * TM;
+	struct tm *TM, s_TM;
 	char tmpdt[40]; /* For temporary storing of dates */
 	int openbraces=0;
 
@@ -327,6 +334,7 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 		} else if (c == '$') switch(*p) {
 			case '{': /* Attribute by Name */
 				decode_attribute(&p, &q, freespace, &openbraces, request, func);
+				break;
 			default:
 				*q++ = c;
 				*q++ = *p;
@@ -347,7 +355,7 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 				q += valuepair2str(q,freespace,pairfind(request->reply->vps,PW_CALLBACK_NUMBER),PW_TYPE_STRING, func);
 				break;
 			case 'd': /* request year */
-				TM = localtime(&request->timestamp);
+				TM = localtime_r(&request->timestamp, &s_TM);
 				strftime(tmpdt,sizeof(tmpdt),"%d",TM);
 				strNcpy(q,tmpdt,freespace);
 				q += strlen(q);
@@ -364,7 +372,7 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 				q += strlen(q);
 				break;
 			case 'm': /* request month */
-				TM = localtime(&request->timestamp);
+				TM = localtime_r(&request->timestamp, &s_TM);
 				strftime(tmpdt,sizeof(tmpdt),"%m",TM);
 				strNcpy(q,tmpdt,freespace);
 				q += strlen(q);
@@ -379,7 +387,7 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 				q += valuepair2str(q,freespace,pairfind(request->packet->vps,PW_CONNECT_INFO),PW_TYPE_STRING, func);
 				break;
 			case 't': /* request timestamp */
-				strNcpy(q,ctime(&request->timestamp),freespace);
+				ctime_r(&request->timestamp, q);
 				q += strlen(q);
 				break;
 			case 'u': /* User name */
@@ -394,7 +402,7 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 				q += strlen(q);
 				break;
 			case 'D': /* request date */
-				TM = localtime(&request->timestamp);
+				TM = localtime_r(&request->timestamp, &s_TM);
 				strftime(tmpdt,sizeof(tmpdt),"%Y%m%d",TM);
 				strNcpy(q,tmpdt,freespace);
 				q += strlen(q);
@@ -411,13 +419,13 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 				q += strlen(q);
 				break;
 			case 'S': /* request timestamp in SQL format*/
-				TM = localtime(&request->timestamp);
+				TM = localtime_r(&request->timestamp, &s_TM);
 				strftime(tmpdt,sizeof(tmpdt),"%Y-%m-%d %H:%M:%S",TM);
 				strNcpy(q,tmpdt,freespace);
 				q += strlen(q);
 				break;
 			case 'T': /* request timestamp */
-				TM = localtime(&request->timestamp);
+				TM = localtime_r(&request->timestamp, &s_TM);
 				strftime(tmpdt,sizeof(tmpdt),"%Y-%m-%d-%H.%M.%S.000000",TM);
 				strNcpy(q,tmpdt,freespace);
 				q += strlen(q);
@@ -433,7 +441,7 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 				q += strlen(q);
 				break;
 			case 'Y': /* request year */
-				TM = localtime(&request->timestamp);
+				TM = localtime_r(&request->timestamp, &s_TM);
 				strftime(tmpdt,sizeof(tmpdt),"%Y",TM);
 				strNcpy(q,tmpdt,freespace);
 				q += strlen(q);
