@@ -1,7 +1,7 @@
 /*
  * rlm_counter.c
  *
- * Version:  $Id: rlm_counter.c,v 1.11 2001/11/09 00:53:10 kkalev Exp $
+ * Version:  $Id: rlm_counter.c,v 1.17 2002/02/25 18:47:55 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 
 #include "radiusd.h"
 #include "modules.h"
@@ -43,8 +44,17 @@
 #	define GDBM_SYNCOPT 0
 #endif
 
+#ifdef GDBM_NOLOCK
+#define GDBM_COUNTER_OPTS (GDBM_SYNCOPT | GDBM_NOLOCK)
+#else
+#define GDBM_COUNTER_OPTS (GDBM_SYNCOPT)
+#endif
 
-static const char rcsid[] = "$Id: rlm_counter.c,v 1.11 2001/11/09 00:53:10 kkalev Exp $";
+#ifndef HAVE_GDBM_FDESC
+#define gdbm_fdesc(foo) (-1)
+#endif
+
+static const char rcsid[] = "$Id: rlm_counter.c,v 1.17 2002/02/25 18:47:55 aland Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -55,7 +65,7 @@ static const char rcsid[] = "$Id: rlm_counter.c,v 1.11 2001/11/09 00:53:10 kkale
  */
 typedef struct rlm_counter_t {
 	char *filename;  /* name of the database file */
-	char *reset;  /* daily, weekly, monthly */
+	char *reset;  /* daily, weekly, monthly, never or user defined */
 	char *key_name;  /* User-Name */
 	char *count_attribute;  /* Acct-Session-Time */
 	char *counter_name;  /* Daily-Session-Time */
@@ -69,6 +79,7 @@ typedef struct rlm_counter_t {
 	time_t last_reset;
 	int dict_attr;  /* attribute number for the counter. */
 	GDBM_FILE gdbm;
+	int fd;
 } rlm_counter_t;
 
 /*
@@ -119,7 +130,10 @@ static int counter_cmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_
 	key_datum.dptr = key_vp->strvalue;
 	key_datum.dsize = key_vp->length;
 
+	if (data->fd >= 0) rad_lockfd(data->fd, sizeof(int));
 	count_datum = gdbm_fetch(data->gdbm, key_datum);
+	if (data->fd >= 0) rad_unlockfd(data->fd, sizeof(int));
+
 	if (count_datum.dptr == NULL) {
 		return -1;
 	}
@@ -133,41 +147,61 @@ static int counter_cmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_
 static int find_next_reset(rlm_counter_t *data, time_t timeval)
 {
 	int ret=0;
-	struct tm *tm=NULL;
+	unsigned int num=1;
+	char last = 0;
+	struct tm *tm, s_tm;
 
-	tm = localtime(&timeval);
+	tm = localtime_r(&timeval, &s_tm);
 	tm->tm_sec = tm->tm_min = 0;
 
-	if (strcmp(data->reset, "hourly") == 0) {
+	if (data->reset == NULL)
+		return -1;
+	if (isdigit(data->reset[0])){
+		unsigned int len=0;
+
+		len = strlen(data->reset);
+		if (len == 0)
+			return -1;
+		last = data->reset[len - 1];
+		if (!isalpha(last))
+			last = 'd';
+		num = atoi(data->reset);
+		DEBUG("rlm_counter: num=%d, last=%c",num,last);
+	}
+	if (strcmp(data->reset, "hourly") == 0 || last == 'h') {
 		/*
 		 *  Round up to the next nearest hour.
 		 */
-		tm->tm_hour++;
+		tm->tm_hour += num;
 		data->reset_time = mktime(tm);
-	} else if (strcmp(data->reset, "daily") == 0) {
+	} else if (strcmp(data->reset, "daily") == 0 || last == 'd') {
 		/*
 		 *  Round up to the next nearest day.
 		 */
 		tm->tm_hour = 0;
-		tm->tm_mday++;
+		tm->tm_mday += num;
 		data->reset_time = mktime(tm);
-	} else if (strcmp(data->reset, "weekly") == 0) {
+	} else if (strcmp(data->reset, "weekly") == 0 || last == 'w') {
 		/*
 		 *  Round up to the next nearest week.
 		 */
 		tm->tm_hour = 0;
-		tm->tm_mday += (7 - tm->tm_wday);
+		tm->tm_mday += (7 - tm->tm_wday) +(7*(num-1));
 		data->reset_time = mktime(tm);
-	} else if (strcmp(data->reset, "monthly") == 0) {
+	} else if (strcmp(data->reset, "monthly") == 0 || last == 'm') {
 		tm->tm_hour = 0;
 		tm->tm_mday = 1;
-		tm->tm_mon++;
+		tm->tm_mon += num;
 		data->reset_time = mktime(tm);
+	} else if (strcmp(data->reset, "never") == 0) {
+		data->reset_time = 0;
 	} else {
 		radlog(L_ERR, "rlm_counter: Unknown reset timer \"%s\"",
-				data->reset);
-		ret=-1;
+			data->reset);
+		return -1;
 	}
+	DEBUG2("rlm_counter: Current Time: %d, Next reset %d", 
+		(int)timeval,(int)data->reset_time);
 
 	return ret;
 }
@@ -275,10 +309,6 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	/*
 	 * Find the attribute for the allowed protocol
 	 */
-	if (data->service_type == NULL) {
-		radlog(L_ERR, "rlm_counter: 'allowed-servicetype' must be set.");
-		exit(0);
-	}
 	if (data->service_type != NULL) {
 		if ((dval = dict_valbyname(PW_SERVICE_TYPE, data->service_type)) == NULL) {
 			radlog(L_ERR, "rlm_counter: Failed to find attribute number for %s",
@@ -300,19 +330,20 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 
 	if (find_next_reset(data,now) == -1)
 		return -1;
-	DEBUG2("rlm_counter: Next reset %d", (int)data->reset_time);
 
 	if (data->filename == NULL) {
 		radlog(L_ERR, "rlm_counter: 'filename' must be set.");
 		exit(0);
 	}
 	data->gdbm = gdbm_open(data->filename, sizeof(int),
-			GDBM_WRCREAT | GDBM_SYNCOPT, 0600, NULL);
+			GDBM_WRCREAT | GDBM_COUNTER_OPTS, 0600, NULL);
 	if (data->gdbm == NULL) {
 		radlog(L_ERR, "rlm_counter: Failed to open file %s: %s",
 				data->filename, strerror(errno));
 		return -1;
 	}
+	if (data->fd >= 0) data->fd = gdbm_fdesc(data->gdbm);
+
 	if (gdbm_setopt(data->gdbm, GDBM_CACHESIZE, &cache_size, sizeof(int)) == -1)
 		radlog(L_ERR, "rlm_counter: Failed to set cache size");
 
@@ -359,12 +390,13 @@ static int counter_accounting(void *instance, REQUEST *request)
 		 *	Open a completely new database.
 		 */
 		data->gdbm = gdbm_open(data->filename, sizeof(int),
-				GDBM_NEWDB | GDBM_SYNCOPT, 0600, NULL);
+				GDBM_NEWDB | GDBM_COUNTER_OPTS, 0600, NULL);
 		if (data->gdbm == NULL) {
 			radlog(L_ERR, "rlm_counter: Failed to open file %s: %s",
 					data->filename, strerror(errno));
 			return RLM_MODULE_FAIL;
 		}
+		if (data->fd >= 0) data->fd = gdbm_fdesc(data->gdbm);
 		if (gdbm_setopt(data->gdbm, GDBM_CACHESIZE, &cache_size, sizeof(int)) == -1)
 			radlog(L_ERR, "rlm_counter: Failed to set cache size");
 	}
@@ -398,6 +430,7 @@ static int counter_accounting(void *instance, REQUEST *request)
 	key_datum.dptr = key_vp->strvalue;
 	key_datum.dsize = key_vp->length;
 
+	if (data->fd >= 0) rad_lockfd(data->fd, sizeof(int));
 	count_datum = gdbm_fetch(data->gdbm, key_datum);
 	if (count_datum.dptr == NULL)
 		counter = 0;
@@ -406,19 +439,39 @@ static int counter_accounting(void *instance, REQUEST *request)
 		free(count_datum.dptr);
 	}
 
-	/*
-	 * if session time < diff then the user got in after the last reset. So add his session time
-	 * else add the diff.
-	 * That way if he logged in at 23:00 and we reset the daily counter at 24:00 and he logged out
-	 * at 01:00 then we will only count one hour (the one in the new day). That is the right thing
-	 */
+	if (count_vp->type == PW_TYPE_DATE) {
+		/*
+		 *	If session time < diff then the user got in after the
+		 *	last reset. So add his session time, otherwise add the
+		 *	diff.
+		 *
+		 *	That way if he logged in at 23:00 and we reset the
+		 *	daily counter at 24:00 and he logged out at 01:00
+		 *	then we will only count one hour (the one in the new
+		 *	day). That is the right thing
+		 */
+		diff = request->timestamp - data->last_reset;
+		counter += (count_vp->lvalue < diff) ? count_vp->lvalue : diff;
 
-	diff = request->timestamp - data->last_reset;
-	counter += (count_vp->lvalue < diff) ? count_vp->lvalue : diff;
+	} else if (count_vp->type == PW_TYPE_INTEGER) {
+		/*
+		 *	Integers get counted, without worrying about
+		 *	reset dates.
+		 */
+		counter += count_vp->lvalue;
+
+	} else {
+		/*
+		 *	The attribute is NOT an integer, just count once
+		 *	more that we've seen it.
+		 */
+		counter++;
+	}
 	count_datum.dptr = (char *) &counter;
 	count_datum.dsize = sizeof(int);
 
 	rcode = gdbm_store(data->gdbm, key_datum, count_datum, GDBM_REPLACE);
+	if (data->fd >= 0) rad_unlockfd(data->fd, sizeof(int));
 	if (rcode < 0) {
 		radlog(L_ERR, "rlm_counter: Failed storing data to %s: %s",
 				data->filename, gdbm_strerror(gdbm_errno));
@@ -470,21 +523,22 @@ static int counter_authorize(void *instance, REQUEST *request)
 		 *	Open a completely new database.
 		 */
 		data->gdbm = gdbm_open(data->filename, sizeof(int),
-				GDBM_NEWDB | GDBM_SYNCOPT, 0600, NULL);
+				GDBM_NEWDB | GDBM_COUNTER_OPTS, 0600, NULL);
 		if (data->gdbm == NULL) {
 			radlog(L_ERR, "rlm_counter: Failed to open file %s: %s",
 					data->filename, strerror(errno));
 			return RLM_MODULE_FAIL;
 		}
+		data->fd = gdbm_fdesc(data->gdbm);
 		if (gdbm_setopt(data->gdbm, GDBM_CACHESIZE, &cache_size, sizeof(int)) == -1)
 			radlog(L_ERR, "rlm_counter: Failed to set cache size");
 	}
 
 
 	/*
-	*      Look for the key.  User-Name is special.  It means
-	*      The REAL username, after stripping.
-	*/
+	 *      Look for the key.  User-Name is special.  It means
+	 *      The REAL username, after stripping.
+	 */
 	DEBUG2("rlm_counter: Entering module authorize code");
 	key_vp = (data->key_attr == PW_USER_NAME) ? request->username : pairfind(request->packet->vps, data->key_attr);
 	if (key_vp == NULL) {
@@ -493,11 +547,11 @@ static int counter_authorize(void *instance, REQUEST *request)
 	}
 
 	/*
-	*      Look for the check item
-	*/
-	
-	if ((dattr = dict_attrbyname(data->check_name)) == NULL)
+	 *      Look for the check item
+	 */
+	if ((dattr = dict_attrbyname(data->check_name)) == NULL) {
 		return ret;
+	}
 	if ((check_vp= pairfind(request->config_items, dattr->attr)) == NULL) {
 		DEBUG2("rlm_counter: Could not find Check item value pair");
 		return ret;
@@ -506,12 +560,13 @@ static int counter_authorize(void *instance, REQUEST *request)
 	key_datum.dptr = key_vp->strvalue;
 	key_datum.dsize = key_vp->length;
 	
+	if (data->fd >= 0) rad_lockfd(data->fd, sizeof(int));
 	count_datum = gdbm_fetch(data->gdbm, key_datum);
+	if (data->fd >= 0) rad_unlockfd(data->fd, sizeof(int));
 	if (count_datum.dptr != NULL){
 		memcpy(&counter, count_datum.dptr, sizeof(int));
 		free(count_datum.dptr);
 	}
-		
 
 	/*
 	 * Check if check item > counter
@@ -519,23 +574,26 @@ static int counter_authorize(void *instance, REQUEST *request)
 	res=check_vp->lvalue - counter;
 	if (res > 0) {
 		/*
-		 * We are assuming that simultaneous-use=1. But even if that does
-		 * not happen then our user could login at max for 2*max-usage-time
-		 * Is that acceptable?
+		 *	We are assuming that simultaneous-use=1. But
+		 *	even if that does not happen then our user
+		 *	could login at max for 2*max-usage-time Is
+		 *	that acceptable?
 		 */
 
 		/*
-		 *  User is allowed, but set Session-Timeout.
-		 *  Stolen from main/auth.c
+		 *	User is allowed, but set Session-Timeout.
+		 *	Stolen from main/auth.c
 		 */
 
 		/*
-		 * If we are near a reset then add the next limit, so that the user will
-		 * not need to login again
+		 *	If we are near a reset then add the next
+		 *	limit, so that the user will not need to
+		 *	login again
 		 */
-
-		if (res >= (data->reset_time - request->timestamp))
+		if (data->reset_time && (
+			res >= (data->reset_time - request->timestamp))) {
 			res += check_vp->lvalue;
+		}
 
 		DEBUG2("rlm_counter: (Check item - counter) is greater than zero");
 		if ((reply_item = pairfind(request->reply->vps, PW_SESSION_TIMEOUT)) != NULL) {
