@@ -1,7 +1,7 @@
 /*
  * rlm_dbm_parser.c :    Create dbm file from plain text
  *
- * Version:     $Id: rlm_dbm_parser.c,v 1.3 2001/11/20 17:15:39 cmiller Exp $
+ * Version:     $Id: rlm_dbm_parser.c,v 1.4 2002/01/14 22:14:27 aland Exp $
  *
  *  This program is is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2 if the
@@ -17,7 +17,7 @@
 
 
 char sccsid[] =
-"$Id: rlm_dbm_parser.c,v 1.3 2001/11/20 17:15:39 cmiller Exp $ sandy module project\n Copyright 2001 Sandy Service\nCopyright 2001 Koulik Andrei";
+"$Id: rlm_dbm_parser.c,v 1.4 2002/01/14 22:14:27 aland Exp $ sandy module project\n Copyright 2001 Sandy Service\nCopyright 2001 Koulik Andrei";
 
 #include "autoconf.h"
 #include <fcntl.h>
@@ -333,12 +333,12 @@ static void sm_usage(void) {
 
 int main(int n,char **argv) {
         
-	char 	*fname = NULL;
-	char	*ofile = NULL;
+	const char *fname = NULL;
+	const char *ofile = NULL;
 	FILE 	*fp;
 	int	print_stat = 1;
 	int 	ch;
-	char	* sm_radius_dir = NULL;
+	const char  *sm_radius_dir = NULL;
 	
 	progname = argv[0];
 
@@ -361,7 +361,7 @@ int main(int n,char **argv) {
 				print_stat = 0;
 				break;
 			case 'v':
-				printf("%s: $Id: rlm_dbm_parser.c,v 1.3 2001/11/20 17:15:39 cmiller Exp $ \n",progname);
+				printf("%s: $Id: rlm_dbm_parser.c,v 1.4 2002/01/14 22:14:27 aland Exp $ \n",progname);
 				exit(0);
 			case 'c':
 				oflags = O_CREAT | O_TRUNC | O_RDWR;
