@@ -1,7 +1,7 @@
 /*
  * rlm_krb5.c	module to authenticate against krb5
  *
- * Version:	$Id: rlm_krb5.c,v 1.10 2001/02/20 20:51:33 pacman Exp $
+ * Version:	$Id: rlm_krb5.c,v 1.11 2002/01/13 11:12:04 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 
-static const char rcsid[] = "$Id: rlm_krb5.c,v 1.10 2001/02/20 20:51:33 pacman Exp $";
+static const char rcsid[] = "$Id: rlm_krb5.c,v 1.11 2002/01/13 11:12:04 fcusack Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -90,10 +90,10 @@ static int krb5_auth(void *instance, REQUEST *request)
 
 	/*
 	 *	We can only authenticate user requests which HAVE
-	 *	a Password attribute.
+	 *	a User-Password attribute.
 	 */
 	if (!request->password) {
-		radlog(L_AUTH, "rlm_krb5: Attribute \"Password\" is required for authentication.");
+		radlog(L_AUTH, "rlm_krb5: Attribute \"User-Password\" is required for authentication.");
 		return RLM_MODULE_INVALID;
 	}
 
@@ -102,7 +102,7 @@ static int krb5_auth(void *instance, REQUEST *request)
 	 *  and not anything else.
 	 */
 	if (request->password->attribute != PW_PASSWORD) {
-		radlog(L_AUTH, "rlm_krb5: Attribute \"Password\" is required for authentication.  Cannot use \"%s\".", request->password->name);
+		radlog(L_AUTH, "rlm_krb5: Attribute \"User-Password\" is required for authentication.  Cannot use \"%s\".", request->password->name);
 		return RLM_MODULE_INVALID;
 	}
 
