@@ -1,7 +1,7 @@
 /*
  * rlm_mschap.c	
  *
- * Version:	$Id: rlm_mschap.c,v 1.7 2001/11/15 18:04:42 3APA3A Exp $
+ * Version:	$Id: rlm_mschap.c,v 1.22 2002/01/21 11:20:42 3APA3A Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -42,6 +42,8 @@
  *  ZARAZA		3APA3A@security.nnov.ru
  */
  
+/*  MPPE support from Takahiro Wagatsuma <waga@sic.shibaura-it.ac.jp> */
+
 #include	"autoconf.h"
 #include	"libradius.h"
 
@@ -54,16 +56,14 @@
 #include	"modules.h"
 
 #include	"des.h"
-#include        "md4.h"
+#include        "md5.h"
 #include	"sha1.h"
 #include	"smbpass.h"
+#include	"rad_assert.h"
 
 #define PW_MSCHAP_RESPONSE	((311 << 16) | 1)
 #define PW_MSCHAP_CHALLENGE	((311 << 16) | 11)
 #define PW_MSCHAP2_RESPONSE	((311 << 16) | 25)
-#define PW_LM_PASSWORD		1057
-#define PW_NT_PASSWORD		1058
-#define PW_SMB_ACCOUNT_CTRL	1059
 
 
 typedef enum {
@@ -77,7 +77,7 @@ static void des_encrypt(const char *szClear, const char *szKey, char *szOut);
 static void mschap(const char *szChallenge, struct smb_passwd * smbPasswd, char *szResponse, int bUseNT);
 static void ntpwdhash (char *szHash, const char *szPassword);
 static void lmpwdhash (char *szHash, const char *szPassword);
-static struct smb_passwd *createsmbpw(char* username, char *password);
+static struct smb_passwd *createsmbpw(struct smb_passwd *pw_buf, const char* username, const char *password);
 static void auth_response(struct smb_passwd * smbPasswd, char *ntresponse,
 		char *peer_challenge, char *auth_challenge,
 		char *response);
@@ -88,6 +88,29 @@ static void mschap2( const char *peer_challenge, const char *auth_challenge,
 static void add_reply(VALUE_PAIR** vp, unsigned char ident,
                 const char* name, const char* value, int len);
 
+static void mppe_add_reply(VALUE_PAIR** vp,
+               const char* name, const char* value, int len);
+
+static void mppe_chap2_gen_keys128(uint8_t *secret,uint8_t *vector,
+                               uint8_t *nt_hash,uint8_t *response,
+                               uint8_t *sendkey,uint8_t *recvkey);
+
+static void mppe_chap2_get_keys128(uint8_t *nt_hashhash,uint8_t *nt_response,
+                               uint8_t *sendkey,uint8_t *recvkey);
+
+static void mppe_GetMasterKey(uint8_t *nt_hashhash,uint8_t *nt_response,
+                       uint8_t *masterkey);
+
+static void mppe_GetAsymmetricStartKey(uint8_t *masterkey,uint8_t *sesskey,
+                               int keylen,int issend);
+
+static void mppe_gen_respkey(uint8_t* secret,uint8_t* vector,
+                       uint8_t* salt,uint8_t* enckey,uint8_t* key);
+
+void md4_calc (unsigned char *, unsigned char *, unsigned int);
+
+
+
 /* 
  *	parity_key takes a 7-byte string in szIn and returns an
  *	8-byte string in szOut.  It inserts a 1 into every 8th bit.
@@ -177,25 +200,23 @@ static void lmpwdhash (char *szHash, const char *szPassword)
  *	if encrypted flag is not set only cleartext password
  *	allowed
  */
-static struct smb_passwd *createsmbpw(char * username, char *password)
+static struct smb_passwd *createsmbpw(struct smb_passwd *pw_buf, const char * username, const char *password)
 {
-	static struct smb_passwd pw_buf;
-	static unsigned char smbpwd[16];
-	static unsigned char smbntpwd[16];
-
-
-	pdb_init_smb(&pw_buf);
-	pw_buf.acct_ctrl = ACB_NORMAL;
-	pw_buf.smb_userid = 0;
-	pw_buf.smb_name = username;
-  
-	if (pw_buf.smb_passwd==NULL && pw_buf.smb_nt_passwd==NULL) {
-		ntpwdhash(smbntpwd, password);
-		lmpwdhash(smbpwd, password);
-		pw_buf.smb_passwd=smbpwd;
-		pw_buf.smb_nt_passwd = smbntpwd;
+	if(pw_buf == NULL) {
+		return NULL;
 	}
-	return &pw_buf;
+	pdb_init_smb(pw_buf);
+	pw_buf->acct_ctrl = ACB_NORMAL;
+	pw_buf->smb_userid = 0;
+	setsmbname(pw_buf,username);
+  
+	if (pw_buf->smb_passwd==NULL && pw_buf->smb_nt_passwd==NULL) {
+		ntpwdhash(pw_buf->smb_nt_passwd_value, password);
+		lmpwdhash(pw_buf->smb_passwd_value, password);
+		pw_buf->smb_passwd = pw_buf->smb_passwd_value;
+		pw_buf->smb_nt_passwd = pw_buf->smb_nt_passwd_value;
+	}
+	return pw_buf;
 }
 
 
@@ -316,6 +337,9 @@ static void auth_response(struct smb_passwd * smbPasswd, char *ntresponse,
 
 struct mschap_instance {
 	int ignore_password;
+	int use_mppe;
+	int require_encryption;
+	int require_strong;
 	char *passwd_file;
 	char *auth_type;
 };
@@ -326,6 +350,12 @@ static CONF_PARSER module_config[] = {
 	 */
 	{ "ignore_password",    PW_TYPE_BOOLEAN,
 	  offsetof(struct mschap_instance,ignore_password), NULL, "no" },
+	{ "use_mppe",    PW_TYPE_BOOLEAN,
+	  offsetof(struct mschap_instance,use_mppe), NULL, "yes" },
+	{ "require_encryption",    PW_TYPE_BOOLEAN,
+	  offsetof(struct mschap_instance,require_encryption), NULL, "no" },
+	{ "require_strong",    PW_TYPE_BOOLEAN,
+	  offsetof(struct mschap_instance,require_strong), NULL, "no" },
 	{ "passwd",   PW_TYPE_STRING_PTR,
 	  offsetof(struct mschap_instance, passwd_file), NULL,  NULL },
 	{ "authtype",   PW_TYPE_STRING_PTR,
@@ -372,12 +402,185 @@ static void add_reply(VALUE_PAIR** vp, unsigned char ident,
 {
 	VALUE_PAIR *reply_attr;
 	reply_attr = pairmake(name, "", T_OP_EQ);
-	*reply_attr->strvalue = ident;
+	if (!reply_attr) {
+		DEBUG("rlm_mschap: add_reply failed to create attribute %s: %s\n", name, librad_errstr);
+		return;
+	}
+
+	reply_attr->strvalue[0] = ident;
 	memcpy(reply_attr->strvalue + 1, value, len);
 	reply_attr->length = len + 1;
 	pairadd(vp, reply_attr);
 }
 
+static void mppe_add_reply(VALUE_PAIR** vp,
+                       const char* name, const char* value, int len)
+{
+       VALUE_PAIR *reply_attr;
+       reply_attr = pairmake(name, "", T_OP_EQ);
+       if (!reply_attr) {
+	       DEBUG("rlm_mschap: mppe_add_reply failed to create attribute %s: %s\n", name, librad_errstr);
+	       return;
+       }
+
+       memcpy(reply_attr->strvalue, value, len);
+       reply_attr->length = len;
+       pairadd(vp, reply_attr);
+}
+
+static void mppe_chap2_gen_keys128(uint8_t *secret,uint8_t *vector,
+                               uint8_t *nt_hash,uint8_t *response,
+                               uint8_t *sendkey,uint8_t *recvkey)
+{
+	uint8_t enckey1[16];
+	uint8_t enckey2[16];
+	uint8_t salt[2];
+	uint8_t nt_hashhash[16];
+
+	md4_calc(nt_hashhash,nt_hash,16);
+
+	mppe_chap2_get_keys128(nt_hashhash,response,enckey1,enckey2);
+
+	salt[0] = (vector[0] ^ vector[1] ^ 0x3A) | 0x80;
+	salt[1] = (vector[2] ^ vector[3] ^ vector[4]);
+
+	mppe_gen_respkey(secret,vector,salt,enckey1,sendkey);
+
+	salt[0] = (vector[0] ^ vector[1] ^ 0x4e) | 0x80;
+	salt[1] = (vector[5] ^ vector[6] ^ vector[7]);
+
+	mppe_gen_respkey(secret,vector,salt,enckey2,recvkey);
+}
+
+static void mppe_chap2_get_keys128(uint8_t *nt_hashhash,uint8_t *nt_response,
+                               uint8_t *sendkey,uint8_t *recvkey)
+{
+       uint8_t masterkey[16];
+
+       mppe_GetMasterKey(nt_hashhash,nt_response,masterkey);
+
+       mppe_GetAsymmetricStartKey(masterkey,sendkey,16,1);
+       mppe_GetAsymmetricStartKey(masterkey,recvkey,16,0);
+}
+
+static uint8_t SHSpad1[40] =
+               { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+                 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+                 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+                 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+
+static uint8_t SHSpad2[40] =
+               { 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2,
+                 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2,
+                 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2,
+                 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2, 0xf2 };
+
+static uint8_t magic1[27] =
+               { 0x54, 0x68, 0x69, 0x73, 0x20, 0x69, 0x73, 0x20, 0x74,
+                 0x68, 0x65, 0x20, 0x4d, 0x50, 0x50, 0x45, 0x20, 0x4d,
+                 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x4b, 0x65, 0x79 };
+
+static uint8_t magic2[84] =
+               { 0x4f, 0x6e, 0x20, 0x74, 0x68, 0x65, 0x20, 0x63, 0x6c, 0x69,
+                 0x65, 0x6e, 0x74, 0x20, 0x73, 0x69, 0x64, 0x65, 0x2c, 0x20,
+                 0x74, 0x68, 0x69, 0x73, 0x20, 0x69, 0x73, 0x20, 0x74, 0x68,
+                 0x65, 0x20, 0x73, 0x65, 0x6e, 0x64, 0x20, 0x6b, 0x65, 0x79,
+                 0x3b, 0x20, 0x6f, 0x6e, 0x20, 0x74, 0x68, 0x65, 0x20, 0x73,
+                 0x65, 0x72, 0x76, 0x65, 0x72, 0x20, 0x73, 0x69, 0x64, 0x65,
+                 0x2c, 0x20, 0x69, 0x74, 0x20, 0x69, 0x73, 0x20, 0x74, 0x68,
+                 0x65, 0x20, 0x72, 0x65, 0x63, 0x65, 0x69, 0x76, 0x65, 0x20,
+                 0x6b, 0x65, 0x79, 0x2e };
+
+static uint8_t magic3[84] =
+               { 0x4f, 0x6e, 0x20, 0x74, 0x68, 0x65, 0x20, 0x63, 0x6c, 0x69,
+                 0x65, 0x6e, 0x74, 0x20, 0x73, 0x69, 0x64, 0x65, 0x2c, 0x20,
+                 0x74, 0x68, 0x69, 0x73, 0x20, 0x69, 0x73, 0x20, 0x74, 0x68,
+                 0x65, 0x20, 0x72, 0x65, 0x63, 0x65, 0x69, 0x76, 0x65, 0x20,
+                 0x6b, 0x65, 0x79, 0x3b, 0x20, 0x6f, 0x6e, 0x20, 0x74, 0x68,
+                 0x65, 0x20, 0x73, 0x65, 0x72, 0x76, 0x65, 0x72, 0x20, 0x73,
+                 0x69, 0x64, 0x65, 0x2c, 0x20, 0x69, 0x74, 0x20, 0x69, 0x73,
+                 0x20, 0x74, 0x68, 0x65, 0x20, 0x73, 0x65, 0x6e, 0x64, 0x20,
+                 0x6b, 0x65, 0x79, 0x2e };
+
+
+static void mppe_GetMasterKey(uint8_t *nt_hashhash,uint8_t *nt_response,
+                       uint8_t *masterkey)
+{
+       uint8_t digest[20];
+       SHA1_CTX Context;
+
+       SHA1Init(&Context);
+       SHA1Update(&Context,nt_hashhash,16);
+       SHA1Update(&Context,nt_response,24);
+       SHA1Update(&Context,magic1,27);
+       SHA1Final(digest,&Context);
+
+       memcpy(masterkey,digest,16);
+}
+
+
+static void mppe_GetAsymmetricStartKey(uint8_t *masterkey,uint8_t *sesskey,
+                               int keylen,int issend)
+{
+       uint8_t digest[20];
+       uint8_t *s;
+       SHA1_CTX Context;
+
+       memset(digest,0,20);
+
+       if(issend) {
+               s = magic3;
+       } else {
+               s = magic2;
+       }
+
+       SHA1Init(&Context);
+       SHA1Update(&Context,masterkey,16);
+       SHA1Update(&Context,SHSpad1,40);
+       SHA1Update(&Context,s,84);
+       SHA1Update(&Context,SHSpad2,40);
+       SHA1Final(digest,&Context);
+
+       memcpy(sesskey,digest,keylen);
+}
+
+static void mppe_gen_respkey(uint8_t* secret,uint8_t* vector,
+                       uint8_t* salt,uint8_t* enckey,uint8_t* key)
+{
+       uint8_t plain[32];
+       uint8_t buf[16];
+       int i;
+       MD5_CTX Context;
+       int slen = strlen(secret);
+
+       memset(key,0,34);
+
+       memset(plain,0,32);
+       plain[0] = 16;
+       memcpy(plain + 1,enckey,16);
+
+       MD5Init(&Context);
+       MD5Update(&Context,secret,slen);
+       MD5Update(&Context,vector,AUTH_VECTOR_LEN);
+       MD5Update(&Context,salt,2);
+       MD5Final(buf,&Context);
+
+       for(i=0;i < 16;i++) {
+               plain[i] ^= buf[i];
+       }
+
+       MD5Init(&Context);
+       MD5Update(&Context,secret,slen);
+       MD5Update(&Context,plain,16);
+       MD5Final(buf,&Context);
+
+       for(i=0;i < 16;i++) {
+               plain[i + 16] ^= buf[i];
+       }
+
+       memcpy(key,salt,2);
+       memcpy(key + 2,plain,32);
+}
 
 
 /*
@@ -391,8 +594,8 @@ static int mschap_authorize(void * instance, REQUEST *request)
 	VALUE_PAIR *challenge = NULL, *response = NULL;
 	VALUE_PAIR *reply_attr;
 	VALUE_PAIR *password = NULL;
-	struct smb_passwd *smbPasswd = NULL;
-	
+	struct smb_passwd smbPasswdValue, *smbPasswd = NULL;
+
 	
 	password = pairfind(request->config_items, PW_PASSWORD);
 	challenge = pairfind(request->packet->vps, PW_MSCHAP_CHALLENGE);
@@ -405,12 +608,16 @@ static int mschap_authorize(void * instance, REQUEST *request)
 		/*  We have nothing related to MS-CHAP or NTLM */
 		return RLM_MODULE_NOOP;
 	}
-	if (password && !inst->ignore_password)
-		smbPasswd = createsmbpw(request->username->strvalue, password->strvalue);
-	else if (inst->passwd_file) {
-		smbPasswd = getsmbfilepwname (inst->passwd_file, request->username->strvalue);
+	if (!request->username || *request->username->strvalue == 0) {
+		/* Usernam must present */
+		return RLM_MODULE_NOOP;
 	}
-	if (!smbPasswd) {
+	if (password && !inst->ignore_password)
+		smbPasswd = createsmbpw(&smbPasswdValue, request->username->strvalue, password->strvalue);
+	else if (inst->passwd_file) {
+		smbPasswd = getsmbfilepwname (&smbPasswdValue, inst->passwd_file, request->username->strvalue);
+	}
+	if (!smbPasswd || !smbPasswd->acct_ctrl&ACB_NORMAL ||smbPasswd->acct_ctrl&ACB_DISABLED) {
 		if(challenge && response){
 			add_reply( &request->reply->vps, *response->strvalue,
 				"MS-CHAP-Error", "E=691 R=1", 9);
@@ -420,31 +627,26 @@ static int mschap_authorize(void * instance, REQUEST *request)
 	if (inst->auth_type){
 		pairdelete(&request->config_items, PW_AUTHTYPE);
 		reply_attr = pairmake("Auth-Type", inst->auth_type, T_OP_EQ);
+		rad_assert(reply_attr != NULL);
 		pairadd(&request->config_items, reply_attr);
 	}
 	if (smbPasswd->smb_passwd){
 		reply_attr = pairmake("LM-Password", "", T_OP_EQ);
+		rad_assert(reply_attr != NULL);
 		reply_attr->length = 16;
 		memcpy(reply_attr->strvalue, smbPasswd->smb_passwd, 16);
 		pairadd(&request->config_items, reply_attr);
 	}
 	if (smbPasswd->smb_nt_passwd){
 		reply_attr = pairmake("NT-Password", "", T_OP_EQ);
+		rad_assert(reply_attr != NULL);
 		reply_attr->length = 16;
 		memcpy(reply_attr->strvalue, smbPasswd->smb_nt_passwd, 16);
 		pairadd(&request->config_items, reply_attr);
 	}
-	if (smbPasswd->acct_ctrl&ACB_DISABLED ||
-		smbPasswd->acct_ctrl&ACB_AUTOLOCK ||
-		!smbPasswd->acct_ctrl&ACB_NORMAL) {
-		if(challenge && response) {
-			add_reply( &request->reply->vps, *response->strvalue,
-				"MS-CHAP-Error", "E=647 R=0", 9);
-		}
-		return RLM_MODULE_USERLOCK;
-	}
 
 	reply_attr = pairmake("SMB-Account-CTRL", "0", T_OP_EQ);
+	rad_assert(reply_attr != NULL);
 	reply_attr->lvalue = smbPasswd->acct_ctrl;
 	pairadd(&request->config_items, reply_attr);
 	return RLM_MODULE_OK;
@@ -471,37 +673,69 @@ static int mschap_authorize(void * instance, REQUEST *request)
  */
 static int mschap_authenticate(void * instance, REQUEST *request)
 {
+#define inst ((struct mschap_instance *)instance)
 	VALUE_PAIR *challenge = NULL, *response = NULL;
 	VALUE_PAIR *password = NULL;
+	VALUE_PAIR *reply_attr;
 	uint8_t calculated[32];
 	uint8_t msch2resp[42];
-	struct smb_passwd smbPasswd, *smbPasswd1 = NULL;
+        uint8_t mppe_sendkey[34];
+        uint8_t mppe_recvkey[34];
+	struct smb_passwd smbPasswd, smbPasswd1Value, *smbPasswd1 = NULL;
 	AUTHTYPE at = NONE;
 	int res = 0;
+	int len = 0;
+	int chap = 0;
 	
 	
 	
-	smbPasswd.smb_name = request->username->strvalue;
+	pdb_init_smb(&smbPasswd);
+	setsmbname(&smbPasswd,request->username->strvalue);
 	password = pairfind(request->config_items, PW_SMB_ACCOUNT_CTRL);
 	if(password){
 		smbPasswd.acct_ctrl = password->lvalue;
 		if (smbPasswd.acct_ctrl&ACB_PWNOTREQ) return RLM_MODULE_OK;
 	}
+        password = pairfind(request->config_items, PW_SMB_ACCOUNT_CTRL_TEXT);
+        if(password) {
+		smbPasswd.acct_ctrl = pdb_decode_acct_ctrl(password->strvalue);
+                if (smbPasswd.acct_ctrl&ACB_PWNOTREQ) return RLM_MODULE_OK;
+        }
 	password = pairfind(request->config_items, PW_LM_PASSWORD);
 	if(password){
-		res++;
-		smbPasswd.smb_passwd = password->strvalue;
+		if(password->length == 16) {
+			smbPasswd.smb_passwd = password->strvalue;
+			res++;
+		}
+		else if(hex2bin(password->strvalue,smbPasswd.smb_passwd_value,16) != 16) {
+			radlog(L_ERR, "rlm_mschap: Invalid LM Password text");
+                } 
+                else {
+			res++;
+			smbPasswd.smb_passwd = smbPasswd.smb_passwd_value;
+		}
+		
 	}
 	password = pairfind(request->config_items, PW_NT_PASSWORD);
 	if(password){
-		res++;
-		smbPasswd.smb_nt_passwd = password->strvalue;
+		if(password->length == 16){
+			smbPasswd.smb_nt_passwd = password->strvalue;
+			res++;
+		}
+		else if(hex2bin(password->strvalue,smbPasswd.smb_nt_passwd_value,16) != 16) {
+			radlog(L_ERR, "rlm_mschap: Invalid NT Password text");
+                }
+                else {
+			smbPasswd.smb_nt_passwd = smbPasswd.smb_nt_passwd_value;
+			res++;
+		}
 	}
 	if (!res) {
-		/*
-		 * We have neither NT nor LM passwords configured
-	 	 */
-	 	return RLM_MODULE_INVALID;
+	/*
+         * We have neither NT nor LM passwords configured
+         */
+		radlog(L_ERR, "rlm_mschap: No LM/NT password configured. Check authorization.");
+		return RLM_MODULE_INVALID;
 	 }
 	
 	/*
@@ -511,20 +745,20 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 	 */
 
 	password = pairfind(request->packet->vps, PW_PASSWORD);
-	if (password) {
+	if (password && request->username && *request->username->strvalue!= 0) {
 		at = CLEARTEXT;
-		smbPasswd1 = createsmbpw(request->username->strvalue, password->strvalue);
+		smbPasswd1 = createsmbpw(&smbPasswd1Value,request->username->strvalue, password->strvalue);
 		if ( (smbPasswd.smb_passwd && !memcmp(smbPasswd1->smb_passwd, smbPasswd.smb_passwd, 16)) ||
 			(smbPasswd.smb_nt_passwd && !memcmp(smbPasswd1->smb_nt_passwd, smbPasswd.smb_nt_passwd, 16)) )
 			return RLM_MODULE_OK;
 		else return RLM_MODULE_REJECT;
 	}
 	else if ( (challenge = pairfind(request->packet->vps, PW_MSCHAP_CHALLENGE)) ){
-
 		/*
 		 *	We need an MS-CHAP-Challenge attribute to calculate
 		 *	the response.
 		 */
+		res = RLM_MODULE_REJECT;
 		if ( (response = pairfind(request->packet->vps, PW_MSCHAP_RESPONSE)) ){
 			if (response->length < 50 || challenge->length < 8) {
 				radlog(L_AUTH, "rlm_mschap: Attribute \"MS-CHAP-Response\" has wrong format.");
@@ -540,20 +774,21 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 			 */
 				mschap(challenge->strvalue, &smbPasswd, calculated, 1);
 				if (memcmp(response->strvalue + 26, calculated, 24) == 0) {
-					return RLM_MODULE_OK;
+					res = RLM_MODULE_OK;
 				}
 			 }
 
-			if (smbPasswd.smb_passwd) {
+			if (res != RLM_MODULE_OK && smbPasswd.smb_passwd) {
 			/*
 			 *	Use LM response.
 			 */
 				mschap(challenge->strvalue, &smbPasswd, 
 					calculated, 0);
 				if (memcmp(response->strvalue + 2, calculated, 24) == 0) {
-					return RLM_MODULE_OK;
+					res = RLM_MODULE_OK;
 				}
 	    		}
+	    		chap = 1;
 		}
 		else if ( (response = pairfind(request->packet->vps, PW_MSCHAP2_RESPONSE)) ){
 			if (response->length < 50 || challenge->length < 16) {
@@ -574,25 +809,80 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 						msch2resp);
 					add_reply( &request->reply->vps, *response->strvalue,
 						"MS-CHAP2-Success", msch2resp, 42);
-					return RLM_MODULE_OK;
+					res = RLM_MODULE_OK;
+					chap = 2;
 				}
 			}
 		}
 		else {
-			radlog(L_AUTH, "rlm_mschap: Response attribute not found");
+			radlog(L_AUTH, "rlm_mschap: Response attribute is not found");
 			return RLM_MODULE_INVALID;
 		}
+		if (res == RLM_MODULE_OK){
+			if (smbPasswd.acct_ctrl&ACB_AUTOLOCK) {
+				add_reply( &request->reply->vps, *response->strvalue,
+					"MS-CHAP-Error", "E=647 R=0", 9);
+				return RLM_MODULE_USERLOCK;
+			}
+
+			/* now create MPPE attributes */
+			if (inst->use_mppe) {
+				if (chap == 1){
+					memset (mppe_sendkey, 0, 32);
+					if (smbPasswd.smb_passwd) 
+						memcpy(mppe_sendkey, smbPasswd.smb_passwd, 8);
+					if (smbPasswd.smb_nt_passwd)
+					/* 
+					   According to RFC 2548 we should send NT hash.
+					   But in practice it doesn't work and we should
+					   send nthashhash instead
+					   If someone have different information please
+					   feel free to feedback.
+
+						memcpy (mppe_sendkey+8,smbPasswd.smb_nt_passwd,16);   
+					*/
+						md4_calc(mppe_sendkey+8, smbPasswd.smb_nt_passwd,16);
+					len = 32;
+					rad_pwencode(mppe_sendkey, &len, 
+						 request->secret, request->packet->vector);
+					mppe_add_reply( &request->reply->vps,
+						"MS-CHAP-MPPE-Keys",mppe_sendkey,len);
+				}
+				else if (chap == 2){
+					mppe_chap2_gen_keys128(request->secret,request->packet->vector,
+						smbPasswd.smb_nt_passwd,
+						response->strvalue + 26,
+						mppe_sendkey,mppe_recvkey);
+					mppe_add_reply( &request->reply->vps,
+						"MS-MPPE-Recv-Key",mppe_recvkey,34);
+					mppe_add_reply( &request->reply->vps,
+						"MS-MPPE-Send-Key",mppe_sendkey,34);
+				}
+				reply_attr = pairmake("MS-MPPE-Encryption-Policy",
+					(inst->require_encryption)? "0x00000002":"0x00000001",
+					T_OP_EQ);
+				rad_assert(reply_attr != NULL);
+				pairadd(&request->reply->vps, reply_attr);
+				reply_attr = pairmake("MS-MPPE-Encryption-Types",
+					(inst->require_strong)? "0x00000004":"0x0000006",
+					T_OP_EQ);
+				rad_assert(reply_attr != NULL);
+				pairadd(&request->reply->vps, reply_attr);
+			
+			}
+			return res;
+		}
 	}
 	
 	add_reply( &request->reply->vps, *response->strvalue,
 		"MS-CHAP-Error", "E=691 R=1", 9);
 	return RLM_MODULE_REJECT;
-
+#undef inst
 }
 
 module_t rlm_mschap = {
   "MS-CHAP",
-  0,				/* type */
+  RLM_TYPE_THREAD_SAFE,				/* type */
   NULL,				/* initialize */
   mschap_instantiate,		/* instantiation */
   {
