@@ -1,7 +1,7 @@
 /*
  * rlm_radutmp.c	
  *
- * Version:	$Id: rlm_radutmp.c,v 1.8 2001/10/15 19:17:47 aland Exp $
+ * Version:	$Id: rlm_radutmp.c,v 1.11 2002/02/22 15:40:54 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -43,36 +43,6 @@
 
 static const char porttypes[] = "ASITX";
 
-/*
- *	Internal wrapper for locking, to minimize the number of ifdef's
- *	in the source.
- *
- *	Lock the utmp file, prefer lockf() over flock()
- */
-static void radutmp_lock(int fd)
-{
-#if defined(F_LOCK) && !defined(BSD)
-	(void)lockf(fd, F_LOCK, LOCK_LEN);
-#else
-	(void)flock(fd, LOCK_EX);
-#endif
-}
-
-/*
- *	Internal wrapper for unlocking, to minimize the number of ifdef's
- *	in the source.
- *
- *	Unlock the utmp file, prefer lockf() over flock()
- */
-static void radutmp_unlock(int fd)
-{
-#if defined(F_LOCK) && !defined(BSD)
-	(void)lockf(fd, F_ULOCK, LOCK_LEN);
-#else
-	(void)flock(fd, LOCK_UN);
-#endif
-}
-
 /*
  *	used for caching radutmp lookups in the accounting component. The
  *	session (checksimul) component doesn't use it, but probably should.
@@ -150,7 +120,7 @@ static int radutmp_zap(struct radutmp_instance *inst, uint32_t nasaddr, time_t t
 		/*
 		 *	Lock the utmp file, prefer lockf() over flock().
 		 */
-		radutmp_lock(fd);
+		rad_lockfd(fd, LOCK_LEN);
 
 		/*
 		 *	Find the entry for this NAS / portno combination.
@@ -171,7 +141,7 @@ static int radutmp_zap(struct radutmp_instance *inst, uint32_t nasaddr, time_t t
 			u.time = t;
 			write(fd, &u, sizeof(u));
 		}
-		close(fd);
+		close(fd);	/* and implicitely release the locks */
 	} else {
 		radlog(L_ERR, "Accounting: %s: %m", inst->radutmp_fn);
 	}
@@ -401,7 +371,7 @@ static int radutmp_accounting(void *instance, REQUEST *request)
 		/*
 		 *	Lock the utmp file, prefer lockf() over flock().
 		 */
-		radutmp_lock(fd);
+		rad_lockfd(fd, LOCK_LEN);
 
 		/*
 		 *	Find the entry for this NAS / portno combination.
@@ -505,7 +475,7 @@ static int radutmp_accounting(void *instance, REQUEST *request)
 				r = -1;
 			}
 		}
-		close(fd);
+		close(fd);	/* and implicitely release the locks */
 	} else {
 		radlog(L_ERR, "Accounting: %s: %m", inst->radutmp_fn);
 		return RLM_MODULE_FAIL;
@@ -532,8 +502,8 @@ static int radutmp_checksimul(void *instance, REQUEST *request)
 	uint32_t	ipno = 0;
 	char		*call_num = NULL;
 	int		rcode;
-	const char *name = (char *)request->username->strvalue;
 	struct radutmp_instance *inst = instance;
+	char		login[256];
 
 	if ((fd = open(inst->radutmp_fn, O_RDWR)) < 0) {
 		if(errno!=ENOENT)
@@ -542,9 +512,14 @@ static int radutmp_checksimul(void *instance, REQUEST *request)
 		return RLM_MODULE_OK;
 	}
 
+	*login = '\0';
+	radius_xlat(login, sizeof(login), inst->username, request, NULL);
+	if(*login == '\0') 
+			return RLM_MODULE_FAIL;
+
 	request->simul_count = 0;
 	while(read(fd, &u, sizeof(u)) == sizeof(u)) {
-		if (strncmp(name, u.login, RUT_NAMESIZE) == 0
+		if (strncmp(login, u.login, RUT_NAMESIZE) == 0
 		    && u.type == P_LOGIN)
 			++request->simul_count;
 	}
@@ -564,17 +539,15 @@ static int radutmp_checksimul(void *instance, REQUEST *request)
 		call_num = vp->strvalue;	
 
 	/*
-	 *	lockf() the file while reading/writing.
+	 *	lock the file while reading/writing.
 	 */
-		radutmp_lock(fd);
+	rad_lockfd(fd, LOCK_LEN);
 
 	request->simul_count = 0;
 	while (read(fd, &u, sizeof(u)) == sizeof(u)) {
-		if (strncmp(name, u.login, RUT_NAMESIZE) == 0
+		if (strncmp(login, u.login, RUT_NAMESIZE) == 0
 		    && u.type == P_LOGIN) {
-			char login[sizeof u.login+1];
 			char session_id[sizeof u.session_id+1];
-			strNcpy(login, u.login, sizeof login);
 			strNcpy(session_id, u.session_id, sizeof session_id);
 
 			/*
@@ -582,10 +555,10 @@ static int radutmp_checksimul(void *instance, REQUEST *request)
 			 *	and we don't want to block everyone else
 			 *	while that's happening.
 			 */
-			radutmp_unlock(fd);
+			rad_unlockfd(fd, LOCK_LEN);
 			rcode = rad_check_ts(u.nas_address, u.nas_port, login, 
 					     session_id);
-			radutmp_lock(fd);
+			rad_lockfd(fd, LOCK_LEN);
 
 			if (rcode == 1) {
 				++request->simul_count;
@@ -603,14 +576,14 @@ static int radutmp_checksimul(void *instance, REQUEST *request)
 				/*
 				 *	False record - zap it.
 				 */
-
-				session_zap(u.nas_address, u.nas_port, login,
+				session_zap(request->packet->sockfd,
+					    u.nas_address, u.nas_port, login,
 					    session_id, u.framed_address,
 					    u.proto, 0);
 			}
 		}
 	}
-	close(fd);
+	close(fd);		/* and implicitely release the locks */
 
 	return RLM_MODULE_OK;
 }
