@@ -1,7 +1,7 @@
 /*
  * sql_mysql.c		SQL Module
  *
- * Version:	$Id: sql_mysql.c,v 1.1 2001/05/14 17:10:21 cmiller Exp $
+ * Version:	$Id: sql_mysql.c,v 1.2 2001/09/28 19:47:10 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -313,24 +313,6 @@ int sql_affected_rows(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 }
 
 
-/*************************************************************************
- *
- *      Function: sql_escape_string
- *
- *      Purpose: Esacpe "'" and any other wierd charactors
- *
- *************************************************************************
- * Unused.  Now provided in rlm_sql main module.
- * But left in here just in case...
- *
-int sql_escape_string(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *to, char *from, int length) {
-
-	mysql_escape_string(to, from, length);
-	return 0;
-}
-*/
-
-
 /* Exported to rlm_sql */
 rlm_sql_module_t rlm_sql_mysql = {
 	"rlm_sql_mysql",
