@@ -1,7 +1,7 @@
 /*
  * sql_postgresql.c		Postgresql rlm_sql driver
  *
- * Version:	$Id: sql_postgresql.c,v 1.16 2001/05/02 23:19:54 cmiller Exp $
+ * Version:	$Id: sql_postgresql.c,v 1.20 2002/02/20 16:22:15 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -167,14 +167,14 @@ int sql_query(SQLSOCK * sqlsocket, SQL_CONFIG *config, char *querystr) {
 			return -1;
 
 		if (strncasecmp("select", querystr, 6) != 0) {
-		/* store the number of affected rows because the sql module
-		 * calls finish_query before it retrieves the number of affected
-		 * rows from the driver */
-		pg_sock->affected_rows = affected_rows(pg_sock->result);
-		return 0;
+			/* store the number of affected rows because the sql module
+			 * calls finish_query before it retrieves the number of affected
+			 * rows from the driver */
+			pg_sock->affected_rows = PQntuples(pg_sock->result);
+			return 0;
 		} else {
 			if ((sql_store_result(sqlsocket, config) == 0)
-					&& (sql_num_fields(sqlsocket, config)>=0))
+					&& (sql_num_fields(sqlsocket, config) >= 0))
 				return 0;
 			else
 				return -1;
@@ -255,8 +255,8 @@ SQL_ROW sql_fetch_row(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 	pg_sock->num_fields = records;
 
 	if ((PQntuples(pg_sock->result) > 0) && (records > 0)) {
-		pg_sock->row = (char **)rad_malloc(records*sizeof(char *)+1);
-		memset(pg_sock->row, '\0', records*sizeof(char *)+1);
+		pg_sock->row = (char **)rad_malloc((records+1)*sizeof(char *));
+		memset(pg_sock->row, '\0', (records+1)*sizeof(char *));
 
 		for (i = 0; i < records; i++) {
 			len = PQgetlength(pg_sock->result, pg_sock->cur_row, i);
@@ -289,7 +289,12 @@ int sql_free_result(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 		PQclear(pg_sock->result);
 		pg_sock->result = NULL;
 	}
+#if 0
+	/*
+	 *  Commented out because it appears to free memory too early.
+	 */
 	free_result_row(pg_sock);
+#endif
 
 	return 0;
 }
