@@ -3,7 +3,7 @@
  *		accounting:     Functions to write radwtmp file.
  *		Also contains handler for "Group".
  *
- * Version:	$Id: rlm_unix.c,v 1.42 2001/11/09 00:53:11 kkalev Exp $
+ * Version:	$Id: rlm_unix.c,v 1.45 2002/02/26 19:22:24 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  * Copyright 2000  Alan Curry <pacman@world.std.com>
  */
-static const char rcsid[] = "$Id: rlm_unix.c,v 1.42 2001/11/09 00:53:11 kkalev Exp $";
+static const char rcsid[] = "$Id: rlm_unix.c,v 1.45 2002/02/26 19:22:24 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -60,6 +60,7 @@ static const char rcsid[] = "$Id: rlm_unix.c,v 1.42 2001/11/09 00:53:11 kkalev E
 #include	"sysutmp.h"
 #include	"cache.h"
 #include	"conffile.h"
+#include	"compat.h"
 
 static char trans[64] =
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
@@ -116,7 +117,6 @@ static int group_inst_explicit;
 
 #if HAVE_GETSPNAM
 #if defined(M_UNIX)
-	typedef struct passwd shadow_pwd_t;
 	static inline const char *get_shadow_name(shadow_pwd_t *spwd) {
 		if (spwd == NULL) return NULL;
 		return (spwd->pw_name);
@@ -126,7 +126,6 @@ static int group_inst_explicit;
 		return (spwd->pw_passwd);
 	}
 #else /* M_UNIX */
-	typedef struct spwd shadow_pwd_t;
 	static inline const char *get_shadow_name(shadow_pwd_t *spwd) {
 		if (spwd == NULL) return NULL;
 		return (spwd->sp_namp);
@@ -428,10 +427,10 @@ static int unix_authenticate(void *instance, REQUEST *request)
 
 	/*
 	 *	We can only authenticate user requests which HAVE
-	 *	a Password attribute.
+	 *	a User-Password attribute.
 	 */
 	if (!request->password) {
-		radlog(L_AUTH, "rlm_unix: Attribute \"Password\" is required for authentication.");
+		radlog(L_AUTH, "rlm_unix: Attribute \"User-Password\" is required for authentication.");
 		return RLM_MODULE_INVALID;
 	}
 
@@ -440,7 +439,7 @@ static int unix_authenticate(void *instance, REQUEST *request)
 	 *  and not anything else.
 	 */
 	if (request->password->attribute != PW_PASSWORD) {
-		radlog(L_AUTH, "rlm_unix: Attribute \"Password\" is required for authentication.  Cannot use \"%s\".", request->password->name);
+		radlog(L_AUTH, "rlm_unix: Attribute \"User-Password\" is required for authentication.  Cannot use \"%s\".", request->password->name);
 		return RLM_MODULE_INVALID;
 	}
 
@@ -506,7 +505,7 @@ static int unix_authenticate(void *instance, REQUEST *request)
 	 *	stand right now.
 	 */
 	if (inst->shadow_file != NULL) {
-		if ((spwd = fgetspnam(inst->shadow_file, name)) == NULL)
+		if ((spwd = fgetspnam(inst->shadow_file, name)) != NULL)
 			encrypted_pass = get_shadow_encrypted_pwd(spwd);
 	} else if ((encrypted_pass == NULL) || (strlen(encrypted_pass) < 10)) {
 		if ((spwd = getspnam(name)) != NULL)
