@@ -1,6 +1,6 @@
 /*
  * x99_rlm.c
- * $Id: x99_rlm.c,v 1.6 2001/11/21 11:30:12 fcusack Exp $
+ * $Id: x99_rlm.c,v 1.20 2002/01/22 22:04:38 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * Copyright 2000  The FreeRADIUS server project
- * Copyright 2001  Google, Inc.
+ * Copyright 2000,2001,2002  The FreeRADIUS server project
+ * Copyright 2001,2002  Google, Inc.
  */
 
 /*
@@ -32,17 +32,16 @@
  */
 
 /*
- * TODO: change maxfail to softfail, hardfail.
  * TODO: support soft PIN? ???
  * TODO: support other than ILP32 (for State)
  */
 
 #include "autoconf.h"
 #include "libradius.h"
-#include "x99.h"
 #include "radiusd.h"
 #include "modules.h"
 #include "conffile.h"
+#include "x99.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -50,34 +49,14 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <openssl/des.h>
 #include <netinet/in.h>	/* htonl() */
 
-static const char rcsid[] = "$Id: x99_rlm.c,v 1.6 2001/11/21 11:30:12 fcusack Exp $";
+static const char rcsid[] = "$Id: x99_rlm.c,v 1.20 2002/01/22 22:04:38 fcusack Exp $";
 
 /* Global data */
 static int rnd_fd;			/* fd for random device           */
 static unsigned char hmac_key[16];	/* to protect State attribute     */
 
-/* struct used for instance data */
-typedef struct x99_token_t {
-    char *pwdfile;	/* file containing user:card_type:key entries      */
-    char *syncdir;	/* dir containing sync mode and state info         */
-    char *chal_text;	/* text to present challenge to user, must have %s */
-    int chal_len;	/* challenge length, min 5 digits                  */
-    int maxdelay;	/* max delay time for response, in seconds         */
-    int maxfail;	/* max number of auth fails before disabling user  */
-    int allow_sync;	/* useful to override pwdfile card_type settings   */
-    int fast_sync;	/* response-before-challenge mode                  */
-    int allow_async;	/* C/R mode allowed?                               */
-    char *chal_req;	/* keyword requesting challenge for fast_sync mode */
-    int ewindow_size;	/* sync mode event window size (right side value)  */
-#if 0
-    int twindow_min;	/* sync mode time window left side                 */
-    int twindow_max;	/* sync mode time window right side                */
-#endif
-} x99_token_t;
-
 /* A mapping of configuration file names to internal variables. */
 static CONF_PARSER module_config[] = {
     { "pwdfile", PW_TYPE_STRING_PTR, offsetof(x99_token_t, pwdfile),
@@ -90,18 +69,30 @@ static CONF_PARSER module_config[] = {
       NULL, "6" },
     { "maxdelay", PW_TYPE_INTEGER, offsetof(x99_token_t, maxdelay),
       NULL, "30" },
-    { "maxfail", PW_TYPE_INTEGER, offsetof(x99_token_t, maxfail),
+    { "softfail", PW_TYPE_INTEGER, offsetof(x99_token_t, softfail),
       NULL, "5" },
+    { "hardfail", PW_TYPE_INTEGER, offsetof(x99_token_t, hardfail),
+      NULL, "0" },
     { "allow_sync", PW_TYPE_BOOLEAN, offsetof(x99_token_t, allow_sync),
       NULL, "yes" },
     { "fast_sync", PW_TYPE_BOOLEAN, offsetof(x99_token_t, fast_sync),
       NULL, "yes" },
-    { "allow_async", PW_TYPE_INTEGER, offsetof(x99_token_t, allow_async),
-      NULL, "0" },
+    { "allow_async", PW_TYPE_BOOLEAN, offsetof(x99_token_t, allow_async),
+      NULL, "no" },
     { "challenge_req", PW_TYPE_STRING_PTR, offsetof(x99_token_t, chal_req),
       NULL, CHALLENGE_REQ },
+    { "resync_req", PW_TYPE_STRING_PTR, offsetof(x99_token_t, resync_req),
+      NULL, RESYNC_REQ },
     { "ewindow_size", PW_TYPE_INTEGER, offsetof(x99_token_t, ewindow_size),
       NULL, "0" },
+    { "mschapv2_mppe", PW_TYPE_INTEGER,
+      offsetof(x99_token_t, mschapv2_mppe_policy), NULL, "2" },
+    { "mschapv2_mppe_bits", PW_TYPE_INTEGER,
+      offsetof(x99_token_t, mschapv2_mppe_types), NULL, "2" },
+    { "mschap_mppe", PW_TYPE_INTEGER,
+      offsetof(x99_token_t, mschap_mppe_policy), NULL, "2" },
+    { "mschap_mppe_bits", PW_TYPE_INTEGER,
+      offsetof(x99_token_t, mschap_mppe_types), NULL, "2" },
 #if 0
     { "twindow_min", PW_TYPE_INTEGER, offsetof(x99_token_t, twindow_min),
       NULL, "0" },
@@ -130,6 +121,9 @@ x99_token_init(void)
 	return -1;
     }
 
+    /* Initialize the password encoding/checking functions. */
+    x99_pwe_init();
+
     return 0;
 }
 
@@ -168,12 +162,18 @@ x99_token_instantiate(CONF_SECTION *conf, void **instance)
 		      "using default of \"%s\"", CHALLENGE_TEXT);
     }
 
-    if (data->maxfail < 1 ) {
-	data->maxfail = 5;
-	radlog(L_ERR, "rlm_x99_token: maxfail must be at least 1, "
+    if (data->softfail < 1 ) {
+	data->softfail = 5;
+	radlog(L_ERR, "rlm_x99_token: softfail must be at least 1, "
 		      "using default of 5");
     }
 
+    if (data->hardfail < 0 ) {
+	data->hardfail = 0;
+	radlog(L_ERR, "rlm_x99_token: hardfail must be at least 1 "
+		      "(or 0 == infinite), using default of 0");
+    }
+
     if (data->fast_sync && !data->allow_sync) {
 	data->fast_sync = 0;
 	radlog(L_INFO, "rlm_x99_token: fast_sync is yes, but allow_sync is no; "
@@ -186,6 +186,30 @@ x99_token_instantiate(CONF_SECTION *conf, void **instance)
 		      "using default of 0", MAX_EWINDOW_SIZE);
     }
 
+    if (data->mschapv2_mppe_policy > 2 || data->mschapv2_mppe_policy < 0) {
+	data->mschapv2_mppe_policy = 2;
+	radlog(L_ERR, "rlm_x99_token: invalid value for mschapv2_mppe, "
+		      "using default of 2");
+    }
+
+    if (data->mschapv2_mppe_types > 2 || data->mschapv2_mppe_types < 0) {
+	data->mschapv2_mppe_types = 2;
+	radlog(L_ERR, "rlm_x99_token: invalid value for mschapv2_mppe_bits, "
+		      "using default of 2");
+    }
+
+    if (data->mschap_mppe_policy > 2 || data->mschap_mppe_policy < 0) {
+	data->mschap_mppe_policy = 2;
+	radlog(L_ERR, "rlm_x99_token: invalid value for mschap_mppe, "
+		      "using default of 2");
+    }
+
+    if (data->mschap_mppe_types != 2) {
+	data->mschap_mppe_types = 2;
+	radlog(L_ERR, "rlm_x99_token: invalid value for mschap_mppe_bits, "
+		      "using default of 2");
+    }
+
 #if 0
     if (data->twindow_max - data->twindow_min > MAX_TWINDOW_SIZE) {
 	data->twindow_min = data->twindow_max = 0;
@@ -225,11 +249,22 @@ x99_token_authorize(void *instance, REQUEST *request)
     unsigned char rawchallenge[MAX_CHALLENGE_LEN];
     char challenge[MAX_CHALLENGE_LEN + 1];	/* +1 for '\0' terminator */
     char *state;
-    int i;
+    int i, rc;
 
     x99_user_info_t user_info;
-    int user_found;
-    int rc;
+    int user_found, auth_type_found;
+    int pwattr;
+    int32_t sflags = 0; /* flags for state */
+    VALUE_PAIR *vp;
+
+    /* Early exit if Auth-Type != x99_token */
+    auth_type_found = 0;
+    if ((vp = pairfind(request->config_items, PW_AUTHTYPE)) != NULL) {
+	auth_type_found = 1;
+	if (strcmp(vp->strvalue, "x99_token")) {
+	    return RLM_MODULE_NOOP;
+	}
+    }
 
     /* The State attribute will be present if this is a response. */
     if (pairfind(request->packet->vps, PW_STATE) != NULL) {
@@ -244,18 +279,9 @@ x99_token_authorize(void *instance, REQUEST *request)
 	return RLM_MODULE_INVALID;
     }
 
-    /* Password attribute required. */
-    if (!request->password) {
-	radlog(L_AUTH, "rlm_x99_token: autz: Attribute \"Password\" is "
-		       "required for authentication.");
-	return RLM_MODULE_INVALID;
-    }
-
-    /* Ensure that we're being passed a plain-text password. */
-    if (request->password->attribute != PW_PASSWORD) {
-	radlog(L_AUTH, "rlm_x99_token: autz: Attribute \"Password\" is "
-		       "required for authentication.  Cannot use \"%s\".",
-		       request->password->name);
+    if ((pwattr = x99_pw_present(request)) == 0) {
+	radlog(L_AUTH, "rlm_x99_token: autz: Attribute \"User-Password\" or "
+		       "equivalent required for authentication.");
 	return RLM_MODULE_INVALID;
     }
 
@@ -263,7 +289,8 @@ x99_token_authorize(void *instance, REQUEST *request)
     user_found = 1;
     if ((rc = x99_get_user_info(inst->pwdfile, request->username->strvalue,
 				&user_info)) == -2) {
-	radlog(L_ERR, "rlm_x99_token: autz: error reading user info");
+	radlog(L_ERR, "rlm_x99_token: autz: error reading user [%s] info",
+	       request->username->strvalue);
 	return RLM_MODULE_FAIL;
     }
     if (rc == -1) {
@@ -278,7 +305,9 @@ x99_token_authorize(void *instance, REQUEST *request)
     if (inst->fast_sync &&
 	((user_info.card_id & X99_CF_SM) || !user_found)) {
 
-	if (!strcmp(request->password->strvalue, inst->chal_req)) {
+	if ((x99_pw_valid(request, inst, pwattr, inst->resync_req, NULL) &&
+		/* Set a bit indicating resync */ (sflags |= htonl(1))) ||
+	    x99_pw_valid(request, inst, pwattr, inst->chal_req, NULL)) {
 	    /*
 	     * Generate a challenge if requested.  We don't test for card
 	     * support [for async] because it's tricky for unknown users.
@@ -298,20 +327,26 @@ x99_token_authorize(void *instance, REQUEST *request)
 	     * the authenticate code to ignore State.  We don't need
 	     * to set a value, /existence/ of the vp is the signal.
 	     */
-	    VALUE_PAIR *vp;
-
 	    if ((vp = paircreate(PW_X99_FAST, PW_TYPE_INTEGER)) == NULL) {
 		radlog(L_ERR|L_CONS, "rlm_x99_token: autz: no memory");
 		return RLM_MODULE_FAIL;
 	    }
 	    pairadd(&request->config_items, vp);
 	    DEBUG("rlm_x99_token: autz: using fast_sync");
+
+	    if (!auth_type_found)
+		pairadd(&request->config_items,
+			pairmake("Auth-Type", "x99_token", T_OP_EQ));
 	    return RLM_MODULE_OK;
 
 	}
     } /* if (fast_sync && card supports sync mode) */
 
 gen_challenge:
+    /* Set the resync bit by default if the user can't request it. */
+    if (!inst->fast_sync)
+	sflags |= htonl(1);
+
     /* Generate a random challenge. */
     if (x99_get_random(rnd_fd, rawchallenge, inst->chal_len) == -1) {
 	radlog(L_ERR, "rlm_x99_token: autz: failed to obtain random data");
@@ -342,13 +377,14 @@ gen_challenge:
 	}
 	now = htonl(now);
 
-	if (x99_gen_state(&state, NULL, challenge, now, hmac_key) != 0) {
+	if (x99_gen_state(&state, NULL, challenge, sflags, now, hmac_key) != 0){
 	    radlog(L_ERR, "rlm_x99_token: autz: failed to generate state");
 	    return RLM_MODULE_FAIL;
 	}
     } else {
+	/* x2 b/c pairmake() string->octet needs even num of digits */
 	state = rad_malloc(3 + inst->chal_len * 2);
-	sprintf(state, "0x%s%s", challenge, challenge);
+	(void) sprintf(state, "0x%s%s", challenge, challenge);
     }
     pairadd(&request->reply->vps, pairmake("State", state, T_OP_EQ));
     free(state);
@@ -358,7 +394,7 @@ gen_challenge:
 	char *u_challenge;	/* challenge with addt'l presentation text */
 
 	u_challenge = rad_malloc(strlen(inst->chal_text) + MAX_CHALLENGE_LEN+1);
-	sprintf(u_challenge, inst->chal_text, challenge);
+	(void) sprintf(u_challenge, inst->chal_text, challenge);
 	pairadd(&request->reply->vps,
 		pairmake("Reply-Message", u_challenge, T_OP_EQ));
 	free(u_challenge);
@@ -371,6 +407,9 @@ gen_challenge:
     request->reply->code = PW_ACCESS_CHALLENGE;
     DEBUG("rlm_x99_token: Sending Access-Challenge.");
 
+    if (!auth_type_found)
+	pairadd(&request->config_items,
+		pairmake("Auth-Type", "x99_token", T_OP_EQ));
     return RLM_MODULE_HANDLED;
 }
 
@@ -383,11 +422,13 @@ x99_token_authenticate(void *instance, REQUEST *request)
 
     x99_user_info_t user_info;
     char *username;
-    int i, failcount;
-    time_t last_async;
+    int i, failcount, pwattr, rc;
+    int32_t sflags = 0; /* flags from state */
+    time_t last_auth;
 
     char challenge[MAX_CHALLENGE_LEN + 1];
-    char e_response[9], u_response[9];		/* expected, user response */
+    char e_response[9];		/* expected response */
+    VALUE_PAIR *add_vps = NULL;
 
     /* User-Name attribute required. */
     if (!request->username) {
@@ -397,26 +438,11 @@ x99_token_authenticate(void *instance, REQUEST *request)
     }
     username = request->username->strvalue;
 
-    /* Password attribute required. */
-    if (!request->password) {
-	radlog(L_AUTH, "rlm_x99_token: auth: Attribute \"Password\" is "
-		       "required for authentication.");
+    if ((pwattr = x99_pw_present(request)) == 0) {
+	radlog(L_AUTH, "rlm_x99_token: auth: Attribute \"User-Password\" or "
+		       "equivalent required for authentication.");
 	return RLM_MODULE_INVALID;
     }
-    /* Early exit for response too long. */
-    if (request->password->length > 8)
-	return RLM_MODULE_REJECT;
-
-    /* Setup u_response. */
-    (void) memset(u_response, 0, sizeof(u_response));
-    (void) memcpy(u_response, request->password->strvalue,
-		  request->password->length);
-    /*
-     * One vendor (at least) uses a '-' in 7 digit display mode.
-     * In case the luser actually types it in, we need to s/-//.
-     */
-    if (u_response[3] == '-')
-	(void) memmove(&u_response[3], &u_response[4], 5);
 
     /* Look up the user's info. */
     if (x99_get_user_info(inst->pwdfile, username, &user_info) != 0) {
@@ -436,7 +462,7 @@ x99_token_authenticate(void *instance, REQUEST *request)
 
 	    /* Extend expected length if state should have been protected. */
 	    if (user_info.card_id & X99_CF_AM)
-		e_length += 4 + 16; /* time + hmac */
+		e_length += 4 + 4 + 16; /* sflags + time + hmac */
 
 	    if (vp->length != e_length) {
 		radlog(L_AUTH, "rlm_x99_token: auth: bad state for [%s]: "
@@ -451,8 +477,9 @@ x99_token_authenticate(void *instance, REQUEST *request)
 	    /* Verify the state. */
 	    (void) memset(challenge, 0, sizeof(challenge));
 	    (void) memcpy(challenge, vp->strvalue, inst->chal_len);
-	    (void) memcpy(&then, vp->strvalue + inst->chal_len, 4);
-	    if (x99_gen_state(NULL, &state, challenge, then, hmac_key) != 0) {
+	    (void) memcpy(&sflags, vp->strvalue + inst->chal_len, 4);
+	    (void) memcpy(&then, vp->strvalue + inst->chal_len + 4, 4);
+	    if (x99_gen_state(NULL,&state,challenge,sflags,then,hmac_key) != 0){
 		radlog(L_ERR, "rlm_x99_token: auth: failed to generate state");
 		return RLM_MODULE_FAIL;
 	    }
@@ -475,31 +502,69 @@ good_state:
 	    /* State is good! */
 
 	} else {
-	    /* This shouldn't happen, authorize code should handle it. */
-	    radlog(L_ERR, "rlm_x99_token: auth: bad state for [%s]: "
-			  "missing", username);
+	    /* This should only happen if the authorize code didn't run. */
+	    radlog(L_ERR|L_CONS, "rlm_x99_token: auth: bad state for [%s]: "
+				 "missing (is x99_token listed in "
+				 "radiusd.conf's authorize stanza?)", username);
 	    return RLM_MODULE_FAIL;
 	}
     } /* if (!fast_sync) */
 
     /*
-     * Check failure count.  We try to "fail secure", but it's not perfect
-     * as we may be able to read the value but not set it.
+     * Check failure count.  We "fail secure".  Note that for "internal"
+     * failures, we don't increment the failcount or last_auth time.
      */
-    if (x99_get_failcount(inst->syncdir, username, &failcount) < 0) {
+    if (x99_get_failcount(inst->syncdir, username, &failcount) != 0) {
 	radlog(L_ERR, "rlm_x99_token: auth: unable to get failure count "
 		      "for [%s]", username);
 	return RLM_MODULE_FAIL;
     }
-    if (failcount >= inst->maxfail) {
+    if (x99_get_last_auth(inst->syncdir, username, &last_auth) != 0) {
+	radlog(L_ERR, "rlm_x99_token: auth: unable to get last auth time "
+		      "for [%s]", username);
+	return RLM_MODULE_FAIL;
+    }
+    if (inst->hardfail && failcount >= inst->hardfail) {
 	radlog(L_AUTH, "rlm_x99_token: auth: %d/%d failed/max authentications "
-		       "for [%s]", failcount, inst->maxfail, username);
+		       "for [%s]", failcount, inst->hardfail, username);
 	if (x99_incr_failcount(inst->syncdir, username) != 0) {
 	    radlog(L_ERR, "rlm_x99_token: auth: unable to increment failure "
 			  "count for locked out user [%s]", username);
 	}
 	return RLM_MODULE_USERLOCK;
     }
+    if (failcount >= inst->softfail) {
+	time_t when;
+	int fcount;
+
+	/*
+	 * Determine the next time this user can authenticate.
+	 *
+	 * Once we hit softfail, we introduce a 1m delay before the user
+	 * can authenticate.  For each successive failed authentication,
+	 * we double the delay time, up to a max of 32 minutes.  While in
+	 * the "delay mode" of operation, all authentication ATTEMPTS are
+	 * considered failures (we don't test if the password is correct).
+	 * Also, each attempt during the delay period restarts the clock.
+	 *
+	 * The advantage of a delay instead of a simple lockout is that an
+	 * attacker can't lock out a user as easily; the user need only wait
+	 * a bit before he can authenticate.
+	 */
+	fcount = failcount - inst->softfail;
+	when = last_auth + (fcount > 5 ? 32 * 60 : (1 << fcount) * 60);
+	if (time(NULL) < when) {
+	    radlog(L_AUTH, "rlm_x99_token: auth: user [%s] auth too soon "
+			   "while delayed, "
+			   "%d/%d failed/softfail authentications",
+			   username, failcount, inst->softfail);
+	    if (x99_incr_failcount(inst->syncdir, username) != 0) {
+		radlog(L_ERR, "rlm_x99_token: auth: unable to increment "
+			      "failure count for delayed user [%s]", username);
+	    }
+	    return RLM_MODULE_USERLOCK;
+	}
+    }
 
     /*
      * Don't bother to check async response if either
@@ -516,6 +581,7 @@ good_state:
 	radlog(L_ERR, "rlm_x99_token: auth: challenge transform failed "
 		      "for [%s]", username);
 	return RLM_MODULE_FAIL;
+	/* NB: last_auth, failcount not updated. */
     }
 
     /* Calculate and test the async response. */
@@ -525,61 +591,64 @@ good_state:
 		      "response for [%s], to challenge %s",
 		      username, challenge);
 	return RLM_MODULE_FAIL;
+	/* NB: last_auth, failcount not updated. */
     }
     DEBUG("rlm_x99_token: auth: [%s], async challenge %s, "
 	  "expecting response %s", username, challenge, e_response);
 
-    if (!strcmp(e_response, u_response)) {
+    if (x99_pw_valid(request, inst, pwattr, e_response, &add_vps)) {
 	/* Password matches.  Is this allowed? */
 	if (!inst->allow_async) {
 	    radlog(L_AUTH, "rlm_x99_token: auth: bad async for [%s]: "
 			   "disallowed by config", username);
-	    return RLM_MODULE_REJECT;
+	    rc = RLM_MODULE_REJECT;
+	    goto return_pw_valid;
+	    /* NB: last_auth, failcount not updated. */
 	}
-	if (x99_get_last_async(inst->syncdir, username, &last_async) != 0) {
-	    radlog(L_ERR, "rlm_x99_token: auth: unable to get last async "
-			  "auth time for [%s]", username);
-	    return RLM_MODULE_FAIL;
-	}
-	if (last_async + inst->maxdelay > time(NULL)) {
+	if (last_auth + inst->maxdelay > time(NULL)) {
 	    radlog(L_AUTH, "rlm_x99_token: auth: bad async for [%s]: "
 			   "too soon", username);
-	    return RLM_MODULE_REJECT;
+	    rc = RLM_MODULE_REJECT;
+	    goto return_pw_valid;
+	    /* NB: last_auth, failcount not updated. */
 	}
 
 	if (user_info.card_id & X99_CF_SM) {
 	    radlog(L_INFO, "rlm_x99_token: auth: [%s] authenticated "
 			   "in async mode", username);
-	    /* Resync the card. */
+	}
+
+	rc = RLM_MODULE_OK;
+	if (ntohl(sflags) & 1) {
+	    /*
+	     * Resync the card.  The sync data doesn't mean anything for
+	     * async-only cards, but we want the side effects of resetting
+	     * the failcount and the last auth time.  We "fail-out" if we
+	     * can't do this, because if we can't update the last auth time,
+	     * we will be open to replay attacks over the lifetime of the
+	     * State attribute (inst->maxdelay).
+	     */
 	    if (x99_get_sync_data(inst->syncdir, username, user_info.card_id,
 				  1, 0, challenge, user_info.keyblock) != 0) {
 		radlog(L_ERR, "rlm_x99_token: auth: unable to get "
 			      "sync data e:%d t:%d for [%s] (for resync)",
 			      1, 0, username);
+		rc = RLM_MODULE_FAIL;
 	    } else if (x99_set_sync_data(inst->syncdir, username, challenge,
 					 user_info.keyblock) != 0) {
 		radlog(L_ERR, "rlm_x99_token: auth: unable to set sync "
 			      "data for [%s] (for resync)", username);
+		rc = RLM_MODULE_FAIL;
+	    }
+	} else {
+	    /* Just update last_auth, failcount. */
+	    if (x99_reset_failcount(inst->syncdir, username) != 0) {
+		radlog(L_ERR, "rlm_x99_token: auth: unable to reset failcount "
+			      "for [%s]", username);
+		rc = RLM_MODULE_FAIL;
 	    }
 	}
-
-	/* Reset counters. */
-	if (x99_reset_failcount(inst->syncdir, username) != 0) {
-	    radlog(L_ERR, "rlm_x99_token: auth: unable to reset "
-			  "failure count for [%s]", username);
-	}
-	if (x99_upd_last_async(inst->syncdir, username) != 0) {
-	    radlog(L_ERR, "rlm_x99_token: auth: unable to update "
-			  "last async time for [%s]", username);
-	    /*
-	     * Up to here, we allowed resync failures to fall through, but
-	     * if we let this one slip by we will be open to replay attacks
-	     * over the lifetime of the State attribute (inst->maxdelay).
-	     */
-	    return RLM_MODULE_FAIL;
-	}
-
-	return RLM_MODULE_OK;
+	goto return_pw_valid;
     } /* if (user authenticated async) */
 
 sync_response:
@@ -593,6 +662,7 @@ sync_response:
 			      "sync data e:%d t:%d for [%s]",
 			      i, 0, username);
 		return RLM_MODULE_FAIL;
+		/* NB: last_auth, failcount not updated. */
 	    }
 
 	    /* Calculate sync response. */
@@ -602,28 +672,35 @@ sync_response:
 			      "sync response e:%d t:%d for [%s], to "
 			      "challenge %s", i, 0, username, challenge);
 		return RLM_MODULE_FAIL;
+		/* NB: last_auth, failcount not updated. */
 	    }
 	    DEBUG("rlm_x99_token: auth: [%s], sync challenge %d %s, "
 		  "expecting response %s", username, i, challenge, e_response);
 
 	    /* Test user-supplied password. */
-	    if (!strcmp(e_response, u_response)) {
-		/* Yay!  User authenticated via sync mode.  Resync. */
+	    if (x99_pw_valid(request, inst, pwattr, e_response, &add_vps)) {
+		/*
+		 * Yay!  User authenticated via sync mode.  Resync.
+		 *
+		 * The same failure/replay issue applies here as in the
+		 * identical code block in the async section, above, with
+		 * the additional problem that a response can be reused
+		 * indefinitely!  (until the sync data is updated)
+		 */
+		rc = RLM_MODULE_OK;
 		if (x99_get_sync_data(inst->syncdir,username,user_info.card_id,
 				      1,0,challenge,user_info.keyblock) != 0) {
 		    radlog(L_ERR, "rlm_x99_token: auth: unable to get "
 				  "sync data e:%d t:%d for [%s] (for resync)",
 				  1, 0, username);
+		    rc = RLM_MODULE_FAIL;
 		} else if (x99_set_sync_data(inst->syncdir, username, challenge,
 					     user_info.keyblock) != 0) {
 		    radlog(L_ERR, "rlm_x99_token: auth: unable to set sync "
 				  "data for [%s] (for resync)", username);
+		    rc = RLM_MODULE_FAIL;
 		}
-		if (x99_reset_failcount(inst->syncdir, username) != 0) {
-		    radlog(L_ERR, "rlm_x99_token: auth: unable to reset "
-				  "failure count for [%s]", username);
-		}
-		return RLM_MODULE_OK;
+		goto return_pw_valid;
 	    }
 
 	} /* for (each slot in the window) */
@@ -635,6 +712,17 @@ sync_response:
 		      "count for user [%s]", username);
     }
     return RLM_MODULE_REJECT;
+
+    /* Must exit here after a successful return from x99_pw_valid(). */
+return_pw_valid:
+
+    /* Handle any vps returned from x99_pw_valid(). */
+    if (rc == RLM_MODULE_OK) {
+	pairadd(&request->reply->vps, add_vps);
+    } else {
+	pairfree(&add_vps);
+    }
+    return rc;
 }
 
 
@@ -648,6 +736,7 @@ x99_token_detach(void *instance)
     free(inst->syncdir);
     free(inst->chal_text);
     free(inst->chal_req);
+    free(inst->resync_req);
     free(instance);
     return 0;
 }
